
import java.io.*;
import sun.audio.*;

interface AudioDevice {
	public int getSampleRate();
	public boolean isStereo();
	public boolean is16Bit();

	public void start();
	public void stop();
}



final class SunAudioDevice extends InputStream implements AudioDevice {

	private InputStream input;

	public SunAudioDevice(InputStream is) throws Exception {
		input = is;
	}

	public final int getSampleRate() {
		return 8000;
	}

	public final boolean isStereo() {
		return false;
	}

	public final boolean is16Bit() {
		return false;
	}

	public final int read() {
		return toUlaw((byte)read());
	}

	public final int read(byte data[], int offset, int length) {
		try {
			input.read(data,offset,length);
		} catch (IOException ioe) {
		}
		for (int i = 0; i < length; i++) {
			data[offset+i] = toUlaw(data[offset+i]);
		}
		return length;
	}

	private static final byte lut[] = {
		0,0,1,1,2,2,2,2,3,3,3,3,3,3,3,3,
		4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
		5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
		5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,6,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
		7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,
	};

	private final byte toUlaw(byte b) {
		return toUlaw((short)((b&0xFF)<<8));
	}

	private final byte toUlaw(short sample) {
		byte sign = (byte) ((sample>>8)&0x80);
		if (sign!=0) sample = (short) -sample;
		byte exponent = lut[(sample>>7)&0xFF];
		byte mantissa = (byte) ((sample>>(exponent+3))&0x0F);
		return (byte) (~(sign | (exponent<<4) | mantissa));
	}


	public final void start() {
		AudioPlayer.player.start(this);
//		AudioPlayer.player.setPriority(Thread.MAX_PRIORITY);
	}

	public final void stop() {
		AudioPlayer.player.stop(this);
	}

}


