
/*
 * Caffeine
 *
 * Martin Storsj 2001
 */


import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;


public final class Caffeine extends Applet implements Runnable, ModListener, KeyListener {

	private int width, height;

	private Graphics graphics;
	private Image bufferImage;
	private Image memoryBufferImage;
	private Graphics bufferGraphics;
	private ColorModel colorModel;
	private MemoryImageSource memoryImage;

	private Image titleImage;
	private Color bgColor = Color.black;

	private int font[], bumpMap[];
	private Image fontImage, bumpMapImage;

	private Thread demoThread;
	private Mod mod;
	private boolean initDone;

	private int time, lastTime;
	private int part;

	public final void init() {
		width = getSize().width;
		height = getSize().height;

		setBackground(bgColor);

		graphics = getGraphics();

		bufferImage = createImage(width,height);
		bufferGraphics = bufferImage.getGraphics();
		bufferGraphics.setColor(bgColor);
		bufferGraphics.fillRect(0,0,bufferImage.getWidth(null),bufferImage.getHeight(null));
		repaint();

		titleImage = Jar.getImageFromJar("title.gif");
		fontImage = Jar.getImageFromJar("font.gif");
		bumpMapImage = Jar.getImageFromJar("bumpmap.jpg");
		MediaTracker mt = new MediaTracker(this);
		mt.addImage(titleImage,0);
		mt.addImage(fontImage,0);
		mt.addImage(bumpMapImage,0);
		while (!mt.checkAll()) {
			try {
				mt.waitForAll();
			} catch (InterruptedException ie) {
			}
		}

		addKeyListener(this);
		initDone = false;
	}

	public final void start() {
//		graphics = getGraphics();
		requestFocus();
		if (demoThread==null) {
			demoThread = new Thread(this);
			demoThread.start();
		}
	}

	public final void stop() {
	}

	public final void destroy() {
//		if (demoThread!=null) demoThread.stop();
	}

	public final void paint(Graphics g) {
		if (bufferImage!=null) g.drawImage(bufferImage,0,0,null);
	}

	public final void update(Graphics g) {
		paint(g);
	}

	public final void repaint() {
		if (bufferImage!=null && graphics!=null)
			graphics.drawImage(bufferImage,0,0,null);
	}

	private final void preCalc(Vector effects) {
		font = makeArray(fontImage);
		for (int i = 0; i < font.length; i++)
			font[i] &= 0x00FFFFFF;
		bumpMap = makeArray(bumpMapImage);
		grayScale(bumpMap);

		for (int i = 0; i < effects.size(); i++)
			((Effect)effects.elementAt(i)).init();
	}

	public final void run() {
//		Thread.currentThread().setPriority(Thread.MAX_PRIORITY);
		initDone = false;

		long demoStartTime = System.currentTimeMillis();

//		puzzle();
		wipeInRight();
		long calcStartTime = System.currentTimeMillis();

		try {
			mod = new Mod(Caffeine.class.getResourceAsStream("caffeine.mod"));
			mod.addListener(this,-1);
		} catch (Exception e) {
			mod = null;
		}
		Vector effects = new Vector();
		Neon neon = new Neon(width, height,null);
		Circular circular = new Circular(width,height,neon);
		Swirl swirl = new Swirl(width,height,neon,1250);
		RotoPlasma rotoplasma = new RotoPlasma(width,height);
//		RotoZoom rotozoom = new RotoZoom(width,height);
//		Plasma plasma = new Plasma(width,height);
//		StarField starfield = new StarField(width,height);
		Glass glass = new Glass(width,height,null,null);
//		Distort distort = new Distort(width,height);
		BumpMap bumpmap = new BumpMap(width,height,null,null);
//		Filter filter = new Filter(width,height);
		Cube cube = new Cube(width,height,0x00FF0000);
//		Fire fire = new Fire(width,height);
//		Tunnel tunnel = new Tunnel(width,height);
//		FadeText fadetext = new FadeText(width,height);
		ShowText showtext = new ShowText(width,height);
		Invert invert = new Invert(width,height,neon);
		Mini mini = new Mini(width,height);
//		Checkers checkers = new Checkers(width,height);
		Growing growing = new Growing(width,height);
		effects.addElement(neon);
		effects.addElement(circular);
		effects.addElement(swirl);
		effects.addElement(rotoplasma);
//		effects.addElement(rotozoom);
//		effects.addElement(plasma);
//		effects.addElement(starfield);
		effects.addElement(glass);
//		effects.addElement(distort);
		effects.addElement(bumpmap);
//		effects.addElement(filter);
		effects.addElement(cube);
//		effects.addElement(fire);
//		effects.addElement(tunnel);
//		effects.addElement(fadetext);
		effects.addElement(showtext);
		effects.addElement(invert);
		effects.addElement(mini);
//		effects.addElement(checkers);
		effects.addElement(growing);
		preCalc(effects);
//		fadetext.setFont(font," !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]",false,16,16,10);
//		fadetext.setText("Caffeine\n\nCode & Graphics: wbs\nMusic: The P",7000,25,25);
		showtext.setFont(font," !\"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]",false,16,16,10,true,0x00FF0000);
		String text[] = {
			"Caffeine",
			"Code & Graphics\n\nwbs",
			"Music\n\nThe P",
			"Thanks to all\nmy friends.\n\nYou know who\nyou are.",
			"Greets to\neverybody at\nAssembly 01!"
		};
		showtext.setText(text,17500);
		System.gc();

		long timeDiff = System.currentTimeMillis() - calcStartTime;
//		System.out.println("Init: "+timeDiff+" msec");
		if (timeDiff < 2500) {
			try {
				Thread.sleep(2500-timeDiff);
			} catch (InterruptedException ie) {
			}
		}

//		wipeOutDown();
		wipeOutRight();

		int buffer[] = new int[width*height];
		int buffer2[] = new int[width*height];
		colorModel = new DirectColorModel(32,0x00FF0000,0x0000FF00,0x000000FF,0);
		memoryImage = new MemoryImageSource(width,height,colorModel,buffer,0,width);
		memoryImage.setAnimated(true);
		memoryImage.setFullBufferUpdates(true);
		bufferImage = memoryBufferImage = createImage(memoryImage);
//		repaint();

/*		String names[] = {
			"Neon, fade",
			"Neon",
			"Neon & Swirl, growing",
			"Swirl",
			"Swirl & Mini, fade",
			"Swirl & Mini",
			"Swirl & Mini & Circular, drop",
			"Circular",
			"Circular & Rotoplasma, fade",
			"Rotoplasma",
			"Rotoplasma & Neon & Invert & Mini, fade",
			"Neon & Invert & Mini",
			"Neon & Circular, fade, Invert & Mini",
			"Circular & Invert & Mini",
			"Circular & Invert & Mini & Neon, fade",
			"Neon",
			"Neon & Cube, fade",
			"Neon & Cube",
			"Neon & Cube & Glass, fade",
			"Neon & Glass",
			"Neon & Glass & Bumpmap, fade",
			"Neon & Bumpmap",
			"Neon & Bumpmap, fade",
			"Neon & ShowText",
			"Neon, fade",
		};
*/
		long startTime = System.currentTimeMillis();
		time = 0;
		int lasttime = 0;
		int frames = 0;
		int step = -1, laststep = 0;
		part = -1;
		boolean done = false;
		boolean modFade = false;

		initDone = true;
		if (mod!=null) mod.play();

		while (!done) {
			time = (int) (System.currentTimeMillis() - startTime);
			if (part<0) continue;
			frames++;

			if (part<2) {
				if (step<=0 && time-lastTime<500) {
					step = 0;
					neon.draw(buffer,time);
					fadeFromBlack(buffer,buffer,time-lastTime,500);
				} else {
					step = 1;
					neon.draw(buffer,time);
				}
			} else if (part<4) {
				if (step<=2 && time-lastTime<750) {
					step = 2;
					neon.draw(buffer,time);
					swirl.draw(buffer2,time);
//					fade(buffer,buffer,buffer2,time-lastTime,500);
//					drop(buffer,buffer2,time-lastTime,500);
					growing.draw(buffer,buffer,buffer2,time-lastTime,750);
				} else {
					step = 3;
					swirl.draw(buffer,time);
				}
			} else if (part<5) {
				if (step<=4 && time-lastTime<500) {
					step = 4;
					swirl.draw(buffer,time);
					System.arraycopy(buffer,0,buffer2,0,buffer.length);
					mini.draw(buffer2);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 5;
					swirl.draw(buffer,time);
					mini.draw(buffer);
				}
			} else if (part<7) {
				if (step<=6 && time-lastTime<500) {
					step = 6;
					swirl.draw(buffer,time);
					mini.draw(buffer);
					circular.draw(buffer2,time);
//					fade(buffer,buffer,buffer2,time-lastTime,500);
					drop(buffer,buffer2,time-lastTime,500);
				} else {
					step = 7;
					circular.draw(buffer,time);
				}
			} else if (part<10) {
				if (step<=8 && time-lastTime<500) {
					step = 8;
					circular.draw(buffer,time);
					rotoplasma.draw(buffer2,time,true);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 9;
					rotoplasma.draw(buffer,time,true);
				}
			} else if (part<12) {
				if (step<=10 && time-lastTime<500) {
					step = 10;
					rotoplasma.draw(buffer,time,true);
					neon.draw(buffer2,time);
					invert.draw(buffer2,time);
					mini.draw(buffer2);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 11;
					neon.draw(buffer,time);
					invert.draw(buffer,time);
					mini.draw(buffer);
				}
			} else if (part<14) {
				if (step<=12 && time-lastTime<500) {
					step = 12;
					neon.draw(buffer,time);
					circular.draw(buffer2,time);
					fade(buffer,buffer,buffer2,time-lastTime,500);
					invert.draw(buffer,time);
					mini.draw(buffer);
				} else {
					step = 13;
					circular.draw(buffer,time);
					invert.draw(buffer,time);
					mini.draw(buffer);
				}
			} else if (part<15) {
				if (step<=14 && time-lastTime<500) {
					step = 14;
					circular.draw(buffer,time);
					invert.draw(buffer,time);
					mini.draw(buffer);
					neon.draw(buffer2,time);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 15;
					neon.draw(buffer,time);
				}
			} else if (part<16) {
				if (step<=16 && time-lastTime<500) {
					step = 16;
					neon.draw(buffer,time);
					System.arraycopy(buffer,0,buffer2,0,buffer.length);
					cube.draw(buffer2,time,false);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 17;
					neon.draw(buffer,time);
					cube.draw(buffer,time,false);
				}
			} else if (part<18) {
				if (step<=18 && time-lastTime<500) {
					step = 18;
					neon.draw(buffer,time);
					glass.draw(buffer2,time,buffer,bumpMap);
					cube.draw(buffer,time,false);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 19;
					neon.draw(buffer,time);
					glass.draw(buffer,time,buffer,bumpMap);
				}
			} else if (part<20) {
				if (step<=20 && time-lastTime<500) {
					step = 20;
					neon.draw(buffer,time);
					bumpmap.draw(buffer2,time,buffer,bumpMap);
					glass.draw(buffer,time,buffer,bumpMap);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 21;
					neon.draw(buffer,time);
					bumpmap.draw(buffer,time,buffer,bumpMap);
				}
			} else if (part<25) {
				if (step<=22 && time-lastTime<500) {
					step = 22;
					neon.draw(buffer2,time);
					bumpmap.draw(buffer,time,buffer2,bumpMap);
					fade(buffer,buffer,buffer2,time-lastTime,500);
				} else {
					step = 23;
					neon.draw(buffer,time);
					showtext.draw(buffer,time);
				}
			} else if (part<26) {
				if (step<=24 && time-lastTime<500) {
					step = 24;
					if (!modFade) {
						if (mod!=null) mod.fadeOut();
						modFade = true;
					}
					neon.draw(buffer,time);
					fadeToBlack(buffer,buffer,time-lastTime,500);
				} else {
					arraySet(buffer,0);
					done = true;
				}
			}

//			mini.draw(buffer);



/*
			if (time < 1250*5.0) {
				swirl.draw(buffer,time+1875,6,false,0);
			} else {
				swirl.draw(buffer,time+1875,6,true,1875+6250);
			}
*/

			if (step != laststep) {
				laststep = step;
//				if (frames>1)
//					System.out.println(names[step-1]+": "+1000*(frames-1)/(time-lasttime)+" fps");
				lasttime = time;
				frames = 1;
			}

			if (demoThread==null) {
				if (mod!=null) mod.stop();
				break;
			}

			memoryImage.newPixels();
			graphics.drawImage(bufferImage,0,0,null);
		}
//		System.out.println(names[step]+": "+1000*frames/(time-lasttime)+" fps");

		arraySet(buffer,0);
		memoryImage.newPixels();
		graphics.drawImage(bufferImage,0,0,null);

//		System.out.println("Total running time: "+(System.currentTimeMillis()-demoStartTime)+" msec");
		demoThread = null;
	}

	public final void channelHit() {
	}
	public final void extraEffect(int effect, int parameter) {
	}
	public final void newPattern(int order) {
		part++;
		lastTime = time;
	}
	public final void newRow(int row, int pattern, int order, int length) {
	}

/*
			if (time < 500) {
				step = 0;
				neon.draw(buffer,time);
				fadeFromBlack(buffer,buffer,time,500);
			} else if (time < 10000) {
				step = 1;
				neon.draw(buffer,time);
			} else if (time < 10500) {
				step = 2;
				neon.draw(buffer,time);
				swirl.draw(buffer2,time);
//				fade(buffer,buffer,buffer2,time-10000,500);
				drop(buffer,buffer2,time-10000,500);
			} else if (time < 20000) {
				step = 3;
				swirl.draw(buffer,time);
			} else if (time < 20500) {
				step = 4;
				swirl.draw(buffer,time);
				circular.draw(buffer2,time);
//				fade(buffer,buffer,buffer2,time-20000,500);
				drop(buffer,buffer2,time-20000,500);
			} else if (time < 30000) {
				step = 5;
				circular.draw(buffer,time);
			} else if (time < 30500) {
				step = 6;
				circular.draw(buffer,time);
				rotoplasma.draw(buffer2,time,false);
				fade(buffer,buffer,buffer2,time-30000,500);
			} else if (time < 40000) {
				step = 7;
				rotoplasma.draw(buffer,time,false);
			} else if (time < 40500) {
				step = 8;
				rotoplasma.draw(buffer,time,false);
				neon.draw(buffer2,time);
				fade(buffer,buffer,buffer2,time-40000,500);
			} else if (time < 42000) {
				step = 9;
				neon.draw(buffer,time);
			} else if (time < 42500) {
				step = 10;
				neon.draw(buffer,time);
				System.arraycopy(buffer,0,buffer2,0,buffer.length);
				cube.draw(buffer2,time,false);
				fade(buffer,buffer,buffer2,time-42000,500);
			} else if (time < 48000) {
				step = 11;
				neon.draw(buffer,time);
				cube.draw(buffer,time,false);
			} else if (time < 48500) {
				step = 12;
				neon.draw(buffer,time);
				glass.draw(buffer2,time,buffer,bumpMap);
				cube.draw(buffer,time,false);
				fade(buffer,buffer,buffer2,time-48000,500);
			} else if (time < 54000) {
				step = 13;
				neon.draw(buffer,time);
				glass.draw(buffer,time,buffer,bumpMap);
			} else if (time < 54500) {
				step = 14;
				neon.draw(buffer,time);
				bumpmap.draw(buffer2,time,buffer,bumpMap);
				glass.draw(buffer,time,buffer,bumpMap);
				fade(buffer,buffer,buffer2,time-54000,500);
			} else if (time < 60000) {
				step = 15;
				neon.draw(buffer,time);
				bumpmap.draw(buffer,time,buffer,bumpMap);
			} else if (time < 60500) {
				step = 16;
				neon.draw(buffer2,time);
				bumpmap.draw(buffer,time,buffer2,bumpMap);
				fade(buffer,buffer,buffer2,time-60000,500);
			} else if (time < 75000) {
				step = 17;
				neon.draw(buffer,time);
				showtext.draw(buffer,time);
			} else if (time < 75500) {
				step = 18;
				neon.draw(buffer,time);
				fadeToBlack(buffer,buffer,time-75000,500);
			}
*/




	private final void puzzle() {
		int delay, delayDecrement, blockSize;
		int width, height;
		int size;
		Point positions[];

		blockSize = 16;

		width = (int) Math.ceil((float)getSize().width/blockSize);
		height = (int) Math.ceil((float)getSize().height/blockSize);

		size = width*height;
		positions = new Point[size];
		for (int y = 0; y < height; y++)
			for (int x = 0; x < width; x++)
				positions[y*width+x] = new Point(x,y);

		delay = 5;
//		delayDecrement = (int) (delay*0.9/height);

		for (int i = 0; i < width*height; i++) {
			int index = rand(size);

			Point p = positions[index];

			for (int j = index; j < size-1; j++)
				positions[j] = positions[j+1];

			size--;

			bufferGraphics.drawImage(titleImage,p.x*blockSize,p.y*blockSize,(p.x+1)*blockSize,(p.y+1)*blockSize,p.x*blockSize,p.y*blockSize,(p.x+1)*blockSize,(p.y+1)*blockSize,bgColor,null);
			repaint();

			try {
				Thread.sleep(delay);
			} catch (InterruptedException ie) {
			}
//			if (i%width == 0) delay -= delayDecrement;
		}
	}

	private final void wipeInRight() {
		int step, stepWidth;

		step = 8;
		stepWidth = (int) Math.ceil((float)width/step);

		for (int i = 0; i < stepWidth+step-1; i++) {
			for (int j = 0; j < step; j++) {
				int x = ((i+1)*step-1-(step+1)*j);
				if (x>=0 && x<width)
					bufferGraphics.drawImage(titleImage,x,0,x+1,height+1,x,0,x+1,height+1,null);
			}
			repaint();
			try {
				Thread.sleep(35);
			} catch (InterruptedException ie) {
			}
		}
	}

	private final void wipeOutDown() {
		int step, stepHeight;

		step = 8;
		stepHeight = (int) Math.ceil((float)height/step);

		bufferGraphics.setColor(bgColor);
		for (int i = 0; i < stepHeight+step-1; i++) {
			for (int j = 0; j < step; j++) {
				int y = ((i+1)*step-1-(step+1)*j);
				bufferGraphics.drawLine(0,y,width,y);
			}
			repaint();
			try {
				Thread.sleep(35);
			} catch (InterruptedException ie) {
			}
		}
	}

	private final void wipeOutRight() {
		int step, stepWidth;

		step = 8;
		stepWidth = (int) Math.ceil((float)width/step);

		bufferGraphics.setColor(bgColor);
		for (int i = 0; i < stepWidth+step-1; i++) {
			for (int j = 0; j < step; j++) {
				int x = ((i+1)*step-1-(step+1)*j);
				bufferGraphics.drawLine(x,0,x,height);
			}
			repaint();
			try {
				Thread.sleep(35);
			} catch (InterruptedException ie) {
			}
		}
	}

	private final void fade(int out[], int in1[], int in2[], int time, int duration) {
		for (int i = 0; i < out.length; i++) {
			int r1, r2, g1, g2, b1, b2;

			r1 = (in1[i]&0x00FF0000)>>16;
			r2 = (in2[i]&0x00FF0000)>>16;
			g1 = (in1[i]&0x0000FF00)>>8;
			g2 = (in2[i]&0x0000FF00)>>8;
			b1 = (in1[i]&0x000000FF);
			b2 = (in2[i]&0x000000FF);

			int r, g, b;

			r = r1 + time*(r2-r1)/duration;
			g = g1 + time*(g2-g1)/duration;
			b = b1 + time*(b2-b1)/duration;

			out[i] = r<<16 | g<<8 | b;
		}
	}

	private final void fadeToBlack(int out[], int in[], int time, int duration) {
		for (int i = 0; i < out.length; i++) {
			int r, g, b;

			r = (duration-time)*((in[i]&0x00FF0000)>>16)/duration;
			g = (duration-time)*((in[i]&0x0000FF00)>>8)/duration;
			b = (duration-time)*(in[i]&0x000000FF)/duration;

			out[i] = r<<16 | g<<8 | b;
		}
	}

	private final void fadeFromBlack(int out[], int in[], int time, int duration) {
		for (int i = 0; i < out.length; i++) {
			int r, g, b;

			r = time*((in[i]&0x00FF0000)>>16)/duration;
			g = time*((in[i]&0x0000FF00)>>8)/duration;
			b = time*(in[i]&0x000000FF)/duration;

			out[i] = r<<16 | g<<8 | b;
		}
	}

	private final void blur(int out[], int in[]) {
		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int rSum = 0;
				int gSum = 0;
				int bSum = 0;
				int c;

				if (y > 0) {
					if (x > 0) {
						c = in[pos-width-1];
						rSum += (c&0x00FF0000)>>16;
						gSum += (c&0x0000FF00)>>8;
						bSum += (c&0x000000FF);
					}

					c = in[pos-width];
					rSum += (c&0x00FF0000)>>16;
					gSum += (c&0x0000FF00)>>8;
					bSum += (c&0x000000FF);

					if (x < width-1) {
						c = in[pos-width+1];
						rSum += (c&0x00FF0000)>>16;
						gSum += (c&0x0000FF00)>>8;
						bSum += (c&0x000000FF);
					}
				}

				if (x > 0) {
					c = in[pos-1];
					rSum += (c&0x00FF0000)>>16;
					gSum += (c&0x0000FF00)>>8;
					bSum += (c&0x000000FF);
				}

				c = in[pos];
				rSum += (c&0x00FF0000)>>16;
				gSum += (c&0x0000FF00)>>8;
				bSum += (c&0x000000FF);

				if (x < width-1) {
					c = in[pos+1];
					rSum += (c&0x00FF0000)>>16;
					gSum += (c&0x0000FF00)>>8;
					bSum += (c&0x000000FF);
				}

				if (y < height-1) {
					if (x > 0) {
						c = in[pos+width-1];
						rSum += (c&0x00FF0000)>>16;
						gSum += (c&0x0000FF00)>>8;
						bSum += (c&0x000000FF);
					}

					c = in[pos+width];
					rSum += (c&0x00FF0000)>>16;
					gSum += (c&0x0000FF00)>>8;
					bSum += (c&0x000000FF);

					if (x < width-1) {
						c = in[pos+width+1];
						rSum += (c&0x00FF0000)>>16;
						gSum += (c&0x0000FF00)>>8;
						bSum += (c&0x000000FF);
					}
				}

				rSum /= 9;
				gSum /= 9;
				bSum /= 9;

				out[pos++] = rSum<<16 | gSum<<8 | bSum;
			}
		}
	}

	public final void drop(int in1[], int in2[], int time, int duration) {
		int y = height*time/duration;
		System.arraycopy(in2,in2.length-y*width,in1,0,y*width);
	}



	public final static void grayScale(int array[]) {
		for (int i = 0; i < array.length; i++) {
			int r, g, b;
			int pixel;
			pixel = array[i];
			r = (pixel&0x00FF0000)>>16;
			g = (pixel&0x0000FF00)>>8;
			b = (pixel&0x000000FF)>>0;
			array[i] = (r+g+b)/3;
		}
	}

	public final static void arrayCopy(int out[], int in[]) {
		int length = (out.length < in.length) ? out.length : in.length;
		for (int i = 0; i < length; i++)
			out[i] = in[i];
	}

	public final static void arraySet(int array[], int value) {
		for (int i = 0; i < array.length; i++)
			array[i] = value;
	}

	public final static int[] makeArray(Image img) {
		int width, height;
		width = img.getWidth(null);
		height = img.getHeight(null);

		int array[] = new int[width*height];
		PixelGrabber pg = new PixelGrabber(img,0,0,width,height,array,0,width);
		try {
			if (!pg.grabPixels()) return null;
		} catch (InterruptedException ie) {
			return null;
		}

		return array;
	}

	private static Random r = new Random();

	public final static int rand(int max) {
		int num;
		num = r.nextInt();
		if (num<0) num = -num;
		num %= max;
		return num;
	}

	public final void keyPressed(KeyEvent ke) {
		if (ke.getKeyCode()==KeyEvent.VK_ESCAPE) {
			if (!initDone) {
				if (demoThread!=null) demoThread.stop();
				if (mod!=null) mod.stop();
				graphics.setColor(bgColor);
				graphics.fillRect(0,0,getSize().width,getSize().height);
			}
			demoThread = null;
		}
	}
	public final void keyTyped(KeyEvent ke) {
	}
	public final void keyReleased(KeyEvent ke) {
	}



	public static Frame frame;
	public static Applet applet;

	public final static void main(String args[]) {
		int width = 320;
		int height = 240;

		frame = new Frame();
		frame.setTitle("Caffeine");
		frame.setBackground(Color.black);
		Canvas c = new Canvas();
		c.setSize(width,height);
		frame.add(c);
		frame.pack();
		frame.addWindowListener(new WindowAdapter() {
			public final void windowClosing(WindowEvent we) {
				applet.stop();
				applet.destroy();
				frame.dispose();
				System.exit(0);
			}
		});
		frame.setResizable(false);
		frame.setVisible(true);
		Insets i = frame.getInsets();
		Dimension d = frame.getToolkit().getScreenSize();
		frame.setLocation((d.width-i.left-i.right-width)/2,(d.height-i.top-i.bottom-height)/2);
		frame.setSize(i.left+i.right+width,i.top+i.bottom+height);
		frame.remove(c);
		applet = new Caffeine();
		applet.addKeyListener(new KeyAdapter() {
			public final void keyPressed(KeyEvent ke) {
				if (ke.getKeyCode()==KeyEvent.VK_ESCAPE) {
					applet.stop();
					applet.destroy();
					frame.dispose();
					System.exit(0);
				}
			}
		});
		frame.add(applet);
		applet.setSize(width,height);
		applet.setBounds(i.left,i.top,width,height);
		applet.init();
		applet.start();
	}

}


abstract class Effect {

	protected int width, height;

	public Effect(int w, int h) {
		width = w;
		height = h;
	}

	public void init() {
	}

	public abstract void draw(int buffer[], int time);

}


interface DataExchangeEffect {
	public Object[] getData();
}

final class Neon extends Effect implements DataExchangeEffect {

	public Neon(int w, int h, DataExchangeEffect dee) {
		super(w,h);
		exchange = dee;
	}

	public final void init() {
		xvar = (512-width)/2;
		yvar = (512-height)/2;

		if (exchange==null) {
			sintable = new short[512];
			for (int i = 0; i < 512; i++)
				sintable[i] = (short) (128+127*Math.sin(2*Math.PI*i/512));

			disttable = new int[512*512];
			int pos = 0;
			for (int y = 0; y < 512; y++)
				for (int x = 0; x < 512; x++)
					disttable[pos++] = (int)((1<<8)*Math.sqrt((x-256)*(x-256)+(y-256)*(y-256)));
		} else {
			Object data[] = exchange.getData();
			sintable = (short[]) data[0];
			disttable = (int[]) data[1];
		}

		palette = createPalette().getPaletteArray();
	}

	public final Object[] getData() {
		Object data[] = new Object[2];
		data[0] = sintable;
		data[1] = disttable;
		return data;
	}

	private DataExchangeEffect exchange;
	private short sintable[];
	private int disttable[];
	private int palette[];
	private int xvar, yvar;

	private final Palette createPalette() {
		Palette p = new Palette(256);

//		for (int i = 0; i < 256; i++)
//			p.setRGB(i,i,i,255);

		for (int i = 0; i < 128; i++)
			p.setRGB(i,0,0,128+i);
//		for (int i = 128; i < 256; i++)
//			p.setRGB(i,i-128,(i-128)<<1,255);
		for (int i = 128; i < 192; i++)
			p.setRGB(i,0,(i-128)<<2,255);
		for (int i = 192; i < 256; i++)
			p.setRGB(i,(i-192)<<2,255,255);

		return p;
	}


	public final void draw(int buffer[], int time) {

		int offsetX1 = (int) (xvar+xvar*Math.sin((double)(3950+time)/900));
		int offsetY1 = (int) (yvar+yvar*Math.cos((double)(2650+time)/750));
		int offsetX2 = (int) (xvar+xvar*Math.sin((double)(4900+time)/850));
		int offsetY2 = (int) (yvar+yvar*Math.cos((double)(1250+time)/1000));
		int offsetX3 = (int) (xvar+xvar*Math.sin((double)(250+time)/750));
		int offsetY3 = (int) (yvar+yvar*Math.cos((double)(2550+time)/950));
		int offsetX4 = (int) (xvar+xvar*Math.sin((double)(2200+time)/1200));
		int offsetY4 = (int) (yvar+yvar*Math.cos((double)(500+time)/1550));


		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				buffer[pos++] = palette[sintable[(
					((
						  disttable[((y+offsetY1)<<9)+x+offsetX1]
						+ disttable[((y+offsetY2)<<9)+x+offsetX2]
						+ disttable[((y+offsetY3)<<9)+x+offsetX3]
						- disttable[((y+offsetY4)<<9)+x+offsetX4]
					)*5)>>8
				)&0x01FF]];
			}
		}
	}

}



final class Circular extends Effect implements DataExchangeEffect {

	public Circular(int w, int h, DataExchangeEffect dee) {
		super(w,h);
		exchange = dee;
	}

	public final void init() {
		xvar = (512-width)/2;
		yvar = (512-height)/2;

		if (exchange==null) {
			sintable = new short[512];
			for (int i = 0; i < 512; i++)
				sintable[i] = (short) (128+127*Math.sin(2*Math.PI*i/512));

			disttable = new int[512*512];
			int pos = 0;
			for (int y = 0; y < 512; y++)
				for (int x = 0; x < 512; x++)
					disttable[pos++] = (int)((1<<8)*Math.sqrt((x-256)*(x-256)+(y-256)*(y-256)));
		} else {
			Object data[] = exchange.getData();
			sintable = (short[]) data[0];
			disttable = (int[]) data[1];
		}

		palette = createPalette().getPaletteArray();
	}

	public final Object[] getData() {
		Object[] data = new Object[2];
		data[0] = sintable;
		data[1] = disttable;
		return data;
	}

	private DataExchangeEffect exchange;
	private short sintable[];
	private int disttable[];
	private int palette[];
	private int xvar, yvar;

	private final Palette createPalette() {
		Palette p = new Palette(256);

		for (int i = 0; i < 256; i++)
			p.setRGB(i,i,i,0);

		return p;
	}


	public final void draw(int buffer[], int time) {

		int offsetX1 = (int) (xvar+xvar*Math.sin((double)(1950+time)/500));
		int offsetY1 = (int) (yvar+yvar*Math.cos((double)(4650+time)/900));
		int offsetX2 = (int) (xvar+xvar*Math.sin((double)(6200+time)/650));
		int offsetY2 = (int) (yvar+yvar*Math.cos((double)(1250+time)/550));

		int size = (int) ((1<<8)*(15+5*Math.sin((double)(1200+time)/700)));

		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int dist1 = disttable[((y+offsetY1)<<9)+x+offsetX1];
				int dist2 = disttable[((y+offsetY2)<<9)+x+offsetX2];
				int c1 = sintable[((dist1*size)>>16)&0x01FF];
				int c2 = sintable[((dist2*size)>>16)&0x01FF];
				if (dist1>dist2)
//				if ((dist1/size1)>(dist2/size2))
					buffer[pos++] = palette[(c1>c2) ? c1 : c2];
				else
					buffer[pos++] = palette[(c1<c2) ? c1 : c2];
//				buffer[pos++] = c1+c2;
			}
		}

	}

}


final class Swirl extends Effect {

	public Swirl(int w, int h, DataExchangeEffect dee, int p) {
		super(w,h);
		// 1250
		exchange = dee;
		period = p;
	}

	public final int getPeriod() {
		return period;
	}

	public final void init() {
		trigFact = (float)Math.PI/period;

		if (exchange==null) {
			sintable = new short[512];
			for (int i = 0; i < 512; i++)
				sintable[i] = (short) (128+127*Math.sin(2*Math.PI*i/512));
		} else {
			Object data[] = exchange.getData();
			sintable = (short[]) data[0];
		}

		int middleX, middleY;
		middleX = width/2;
		middleY = height/2;

		int pos;

		angletable = new int[width*height];
		pos = 0;
		for (int y = 0; y < height; y++)
			for (int x = 0; x < width; x++)
				angletable[pos++] = (int) ((1<<16)*512*Math.atan2((y-middleY),(x-middleX))/(2*Math.PI));
		disttable = new int[width*height];
		pos = 0;
		for (int y = 0; y < height; y++)
			for (int x = 0; x < width; x++)
				disttable[pos++] = (int) ((1<<16)*Math.sqrt((x-middleX)*(x-middleX)+(y-middleY)*(y-middleY))/10);

		palette = createPalette().getPaletteArray();
	}

	private DataExchangeEffect exchange;
	private int period;
	private float trigFact;
	private short sintable[];
	private int angletable[];
	private int disttable[];
	private int palette[];

	private final Palette createPalette() {
		Palette p = new Palette(256);

		for (int i = 0; i < 128; i++)
			p.setRGB(i,0,0,(128-i)<<1);
//			p.setRGB(i,(128-i)<<1,0,0);
		for (int i = 128; i < 256; i++)
			p.setRGB(i,0,(i-128)<<1,0);
//			p.setRGB(i,(i-128)<<1,(i-128)<<1,0);

//		for (int i = 0; i < 256; i++)
//			p.setRGB(i,(255-i),i,0);

		return p;
	}

	public final void draw(int buffer[], int time) {
		draw(buffer,time,6,false,0);
	}

	public final void draw(int buffer[], int time, int multiplier, boolean direction, int turnTime) {

//		int timeOffset = (int) (start*period);
//		int timeOffset = start;

//		float curvyness = (float) (50*Math.sin((float)time/1000));
		int curvyness = (int) ((1<<8)*50*Math.sin((float)trigFact*time));
		int spin;
//		if (time < 6.5*Math.PI*1000)
//			spin = (int) (time/1.5);
//		else
//			spin = (int) ((2*6.5*Math.PI*1000-time)/1.5);

		if (!direction)
			spin = (int) (time/1.0);
		else
			spin = (int) ((2*turnTime-time)/1.0);


/*		int multiplier;
//		if (time < 1*Math.PI*1000)
//			multiplier = 1;
		if (time < 2*Math.PI*1000)
			multiplier = 1;
		else if (time < 4*Math.PI*1000)
			multiplier = 3;
		else if (time < 9*Math.PI*1000)
			multiplier = 6;
		else
			multiplier = 12;
*/
		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				buffer[pos] = palette[sintable[(spin + ((multiplier*angletable[pos]+curvyness*(disttable[pos]>>8))>>16))&0x01FF]];
				pos++;
			}
		}

	}

}


final class RotoPlasma extends Effect {

	public RotoPlasma(int w, int h) {
		super(w,h);
	}

	public final void init() {

		angleA = (float)Math.atan2(-height,-width);
		angleB = (float)Math.atan2(-height,width);
		angleC = (float)Math.atan2(height,-width);

		texture1 = new byte[512*512];
		texture2 = new byte[512*512];

		int pos;

/*		pos = 0;
		for (int y = 0; y < 256; y++) {
			for (int x = 0; x < 256; x++) {
				int c1, c2;
				c1 = 64 + (int)(63*Math.sin(10+x/(85+30*Math.sin((float)y/55)))*Math.cos(15+y/(75+20*Math.cos((float)x/50))));
				c2 = 64 + (int)(63*Math.sin(x/(75+25*Math.cos((float)y/45)))*Math.cos(y/(80+15*Math.sin((float)x/55))));
				texture1[y*512+x] = c1;
				texture1[(y+1)*512-x-1] = c1;
				texture1[(512-y-1)*512+x] = c1;
				texture1[(512-y)*512-x-1] = c1;
				texture2[y*512+x] = c2;
				texture2[(y+1)*512-x-1] = c2;
				texture2[(512-y-1)*512+x] = c2;
				texture2[(512-y)*512-x-1] = c2;
			}
		}
*/
		pos = 0;
		for (int y = 0; y < 512; y++) {
			for (int x = 0; x < 512; x++) {
//				texture1[pos] = 64 + (int)(63*Math.sin(10+x/(85+30*Math.sin((float)y/55)))*Math.cos(15+y/(75+20*Math.cos((float)x/50))));
//				texture2[pos] = 64 + (int)(63*Math.sin(x/(75+25*Math.cos((float)y/45)))*Math.cos(y/(80+15*Math.sin((float)x/55))));
				texture1[pos] = (byte)(64 + (int)(63*Math.sin(10+x/(42+15*Math.sin((float)y/27)))*Math.cos(15+y/(38+10*Math.cos((float)x/25)))));
				texture2[pos] = (byte)(64 + (int)(63*Math.sin(x/(37+13*Math.cos((float)y/23)))*Math.cos(y/(40+8*Math.sin((float)x/28)))));
				pos++;
			}
		}

		palette = createPalette().getPaletteArray();
	}

	private float angleA, angleB, angleC;
	private byte texture1[];
	private byte texture2[];
	private int palette[];

	private final Palette createPalette() {
		Palette p = new Palette(256);

		for (int i = 0; i < 256; i++)
			p.setRGB(i,(255-i),i,0);

		return p;
	}

	public final void draw(int buffer[], int time) {
		draw(buffer,time,true);
	}

	public final void draw(int buffer[], int time, boolean bilinear) {

//			float radius = (float)(150+120*Math.sin((float)time/2000));
			float radius = (float)(75+45*Math.sin((float)time/2000));
//			float radius = (float) 90;
			float angle;
			if (time < 15000)
				angle = (float) Math.PI*time/3000;
			else
				angle = (float) Math.PI*(30000-time)/3000;
			float centerX = (float)(256+10*Math.cos((float)time/1000));
			float centerY = (float)(256+10*Math.sin(10+(float)time/800));

			float radiusA = (float) ((90+10*Math.sin(2+(float)time/1000))/100);
			float radiusB = (float) ((80+20*Math.sin(3+(float)time/1200))/100);
			float radiusC = (float) ((85+15*Math.sin(4+(float)time/850))/100);

			int ax = (int)((centerX+radius*radiusA*Math.cos(angleA+angle))*(1<<16));
			int ay = (int)((centerY+radius*radiusA*Math.sin(angleA+angle))*(1<<16));
			int bx = (int)((centerX+radius*radiusB*Math.cos(angleB+angle))*(1<<16));
			int by = (int)((centerY+radius*radiusB*Math.sin(angleB+angle))*(1<<16));
			int cx = (int)((centerX+radius*radiusC*Math.cos(angleC+angle))*(1<<16));
			int cy = (int)((centerY+radius*radiusC*Math.sin(angleC+angle))*(1<<16));

			int dxdx = (bx-ax)/width;
			int dydx = (by-ay)/width;
			int dxdy = (cx-ax)/height;
			int dydy = (cy-ay)/height;

			int offsetX1 = (int)(120*Math.sin((float)(3000+time)/2000)*(1<<16));
			int offsetY1 = (int)(120*Math.cos((float)(4000+time)/3000)*(1<<16));
			int offsetX2 = (int)(120*Math.cos((float)(11000+time)/2500)*(1<<16));
			int offsetY2 = (int)(120*Math.sin((float)(13000+time)/2800)*(1<<16));

			int pos = 0;
			for (int y = 0; y < height; y++) {
				cx = ax;
				cy = ay;
				for (int x = 0; x < width; x++) {
//					buffer[pos++] = texture1[(((cy+offsetY1)>>7)&0x03FE00)+(((cx+offsetX1)>>16)&0x01FF)]+texture2[(((cy+offsetY2)>>7)&0x03FE00)+(((cx+offsetX2)>>16)&0x01FF)];
//					buffer[pos++] = texture1[(((cy+offsetY1)>>6)&0x0FFC00)+(((cx+offsetX1)>>16)&0x03FF)]+texture2[(((cy+offsetY2)>>6)&0x0FFC00)+(((cx+offsetX2)>>16)&0x03FF)];

					if (!bilinear) {
						buffer[pos++] = palette[texture1[(((cy+offsetY1)>>7)&0x03FE00)+((cx+offsetX1)>>16)]+texture2[(((cy+offsetY2)>>7)&0x03FE00)+((cx+offsetX2)>>16)]];
					} else {
						int px11, px12, px13, px14, px21, px22, px23, px24;
						px11 = texture1[(((cy+offsetY1)>>7)&0x03FE00)+((cx+offsetX1)>>16)];
						px12 = texture1[(((cy+offsetY1)>>7)&0x03FE00)+((cx+offsetX1)>>16)+1];
						px13 = texture1[(((cy+offsetY1)>>7)&0x03FE00)+0x01FF+((cx+offsetX1)>>16)];
						px14 = texture1[(((cy+offsetY1)>>7)&0x03FE00)+0x01FF+((cx+offsetX1)>>16)+1];
						px21 = texture2[(((cy+offsetY2)>>7)&0x03FE00)+((cx+offsetX2)>>16)];
						px22 = texture2[(((cy+offsetY2)>>7)&0x03FE00)+((cx+offsetX2)>>16)+1];
						px23 = texture2[(((cy+offsetY2)>>7)&0x03FE00)+0x01FF+((cx+offsetX2)>>16)];
						px24 = texture2[(((cy+offsetY2)>>7)&0x03FE00)+0x01FF+((cx+offsetX2)>>16)+1];
						int fx1, fx2, fy1, fy2;
						fx1 = (cx+offsetX1)&0xFFFF;
						fy1 = (cy+offsetY1)&0xFFFF;
						fx2 = (cx+offsetX2)&0xFFFF;
						fy2 = (cy+offsetY2)&0xFFFF;
						buffer[pos++] = palette[(
							px11*(((0xFFFF-fx1)*(0xFFFF-fy1))>>>16) +
							px12*(((fx1)*(0xFFFF-fy1))>>>16) +
							px13*(((0xFFFF-fx1)*(fy1))>>>16) +
							px14*(((fx1)*(fy1))>>>16) +
							px21*(((0xFFFF-fx2)*(0xFFFF-fy2))>>>16) +
							px22*(((fx2)*(0xFFFF-fy2))>>>16) +
							px23*(((0xFFFF-fx2)*(fy2))>>>16) +
							px24*(((fx2)*(fy2))>>>16)
						)>>>16];
					}

					cx += dxdx;
					cy += dydx;
				}
				ax += dxdy;
				ay += dydy;
			}

	}

}


final class RotoZoom extends Effect {

	public RotoZoom(int w, int h) {
		super(w,h);
	}

	public final void init() {

		angleA = (float)Math.atan2(-height,-width);
		angleB = (float)Math.atan2(-height,width);
		angleC = (float)Math.atan2(height,-width);


		Palette p = new Palette(4);
		p.setRGB(0,0,0,0);
		p.setRGB(1,0,255,0);
		p.setRGB(2,0,0,255);
		p.setRGB(3,255,0,0);
		palette = p.getPaletteArray();
	}

	private float angleA, angleB, angleC;
	private int palette[];

	private final byte texture[] = {
/*		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		1,0,2,2,0,0,0,0,0,0,0,0,0,2,2,0,
		1,0,2,2,2,0,0,0,0,0,0,0,2,2,2,0,
		1,0,0,2,2,2,0,0,0,0,0,2,2,2,0,0,
		1,0,0,0,2,2,2,0,0,0,2,2,2,0,0,0,
		1,0,0,0,0,2,2,2,0,2,2,2,0,0,0,0,
		1,0,0,0,0,0,2,2,2,2,2,0,0,0,0,0,
		1,0,0,0,0,0,0,2,2,2,0,0,0,0,0,0,
		1,0,0,0,0,0,2,2,2,2,2,0,0,0,0,0,
		1,0,0,0,0,2,2,2,0,2,2,2,0,0,0,0,
		1,0,0,0,2,2,2,0,0,0,2,2,2,0,0,0,
		1,0,0,2,2,2,0,0,0,0,0,2,2,2,0,0,
		1,0,2,2,2,0,0,0,0,0,0,0,2,2,2,0,
		1,0,2,2,0,0,0,0,0,0,0,0,0,2,2,0,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
*/
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,
		1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,
		1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,
		1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,
		1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,
		1,1,0,0,0,0,0,2,2,2,0,0,0,0,0,1,1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,
		1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,
		1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,
		1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,
		1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,
		1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,
		1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,
		1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,
		1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,
		1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,
		1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,1,1,0,0,0,0,0,2,2,2,0,0,0,0,0,1,
		1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,
		1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,
		1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,
		1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,
		1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
	};

	public final void draw(int buffer[], int time) {

			float radius = (float)(150+120*Math.sin((float)time/2000));
//			float radius = (float)(75+45*Math.sin((float)time/2000));
//			float radius = (float) 90;
			float angle;
			if (time < 15000)
				angle = (float) Math.PI*time/3000;
			else
				angle = (float) Math.PI*(30000-time)/3000;
			float centerX = (float)(10*Math.cos((float)time/1000));
			float centerY = (float)(10*Math.sin(10+(float)time/800));

			float radiusA = (float) ((90+25*Math.sin(2+(float)time/1000))/100);
			float radiusB = (float) ((80+30*Math.sin(3+(float)time/1200))/100);
			float radiusC = (float) ((85+20*Math.sin(4+(float)time/850))/100);

			int ax = (int)((centerX+radius*radiusA*Math.cos(angleA+angle))*(1<<16));
			int ay = (int)((centerY+radius*radiusA*Math.sin(angleA+angle))*(1<<16));
			int bx = (int)((centerX+radius*radiusB*Math.cos(angleB+angle))*(1<<16));
			int by = (int)((centerY+radius*radiusB*Math.sin(angleB+angle))*(1<<16));
			int cx = (int)((centerX+radius*radiusC*Math.cos(angleC+angle))*(1<<16));
			int cy = (int)((centerY+radius*radiusC*Math.sin(angleC+angle))*(1<<16));

			int dxdx = (bx-ax)/width;
			int dydx = (by-ay)/width;
			int dxdy = (cx-ax)/height;
			int dydy = (cy-ay)/height;


			int pos = 0;
			for (int y = 0; y < height; y++) {
				cx = ax;
				cy = ay;
				for (int x = 0; x < width; x++) {
//					buffer[pos++] = texture[((cy>>8)&0xFF00)+((cx>>16)&0xFF)];
					buffer[pos++] = palette[texture[((cy>>11)&0x03E0)+((cx>>16)&0x1F)]];
					cx += dxdx;
					cy += dydx;
				}
				ax += dxdy;
				ay += dydy;
			}

	}

}


final class Plasma extends Effect {

	public Plasma(int w, int h) {
		super(w,h);
	}

	public final void init() {

		texture1 = new byte[width*2*height*2];
		texture2 = new byte[width*2*height*2];

		int pos = 0;
		for (int y = 0; y < height*2; y++) {
			for (int x = 0; x < width*2; x++) {
//				texture1[pos] = 64 + (int)(63*Math.sin(Math.sqrt((x-width)*(x-width)+(y-height)*(y-height))/17));
//				texture1[pos] = 64 + (int)(63*Math.sin(10+x/(75+25*Math.sin((float)y/35)))*Math.cos(15+y/(65+35*Math.cos((float)x/65))));
//				texture1[pos] = 64 + (int)(63*Math.sin(10+x/(95+35*Math.sin((float)y/35)))*Math.cos(15+y/(75+35*Math.cos((float)x/35))));
				texture1[pos] = (byte)(64 + 63*Math.sin(10+x/(85+30*Math.sin((float)y/55)))*Math.cos(15+y/(75+20*Math.cos((float)x/50))));


				texture2[pos] = (byte)(64 + 63*Math.sin(x/(75+25*Math.cos((float)y/45)))*Math.cos(y/(80+15*Math.sin((float)x/55))));

				pos++;
			}
		}

		palette = createPalette().getPaletteArray();
	}

	private byte texture1[];
	private byte texture2[];
	private int palette[];

	private final Palette createPalette() {
		Palette p = new Palette(256);

		for (int i = 0; i < 256; i++)
			p.setRGB(i,(255-i),i,0);

		return p;
	}

	public final void draw(int buffer[], int time) {

			int offsetX1 = width/2+(int)((width/2)*Math.sin((float)(6000+time)/2000));
			int offsetY1 = height/2+(int)((height/2)*Math.cos((float)(7000+time)/3000));
			int offsetX2 = width/2+(int)((width/2)*Math.cos((float)(14000+time)/2500));
			int offsetY2 = height/2+(int)((height/2)*Math.sin((float)(16000+time)/2800));

			int pos = 0;
			for (int y = 0; y < height; y++)
				for (int x = 0; x < width; x++)
					buffer[pos++] = palette[texture1[(y+offsetY1)*width*2+x+offsetX1] + texture2[(y+offsetY2)*width*2+x+offsetX2]];

	}

}


final class StarField extends Effect {

	public StarField(int w, int h) {
		super(w,h);
	}

	public final void init() {

		stars = new Star[40];

		for (int i = 0; i < stars.length; i++) {

			stars[i] = new Star();

			stars[i].plane = Caffeine.rand(3);
			switch (stars[i].plane) {
				case 0:
					stars[i].color = 0x00808080;
					break;
				case 1:
					stars[i].color = 0x00C0C0C0;
					break;
				case 2:
					stars[i].color = 0x00FFFFFF;
					break;
			}
			stars[i].x = Caffeine.rand(width*(1<<16));
			stars[i].y = Caffeine.rand(height);
		}

	}

	private final class Star {
		public int x;
		public int y;
		public int plane;
		public int color;
	}

	private Star stars[];
	private int lasttime = 0;

	public final void draw(int buffer[], int time) {
		draw(buffer,time,true);
	}

	public final void draw(int buffer[], int time, boolean clear) {

		int difftime = time - lasttime;
		lasttime = time;

		if (clear) Caffeine.arraySet(buffer,0);

		for (int j = 0; j < stars.length; j++) {
//			buffer[(int)stars[j].y*width+(int)(stars[j].x>>16)] = 0;

			stars[j].x -= ((stars[j].plane+1)<<16)*difftime/25;
// *difftime/60;
			if (stars[j].x<0) {
				stars[j].x = (width-1)<<16;
				stars[j].y = Caffeine.rand(height);
			}
			buffer[(int)stars[j].y*width+(int)(stars[j].x>>16)] = stars[j].color;
		}

	}

}


final class Glass extends Effect {

	public Glass(int w, int h, Image bg, Image m) {
		super(w,h);
		bgImage = bg;
		mapImage = m;
	}

	public final void init() {

		if (mapImage!=null) {
			map = Caffeine.makeArray(mapImage);
			Caffeine.grayScale(map);
		}
		if (bgImage!=null) {
			image = Caffeine.makeArray(bgImage);
			imagewidth = bgImage.getWidth(null);
			xvar = (bgImage.getWidth(null)-width)/2;
			yvar = (bgImage.getHeight(null)-height)/2;
		}

	}

	private Image mapImage, bgImage;
	private int map[];

	private int image[];
	private int imagewidth;

	private int xvar, yvar;

	public final void draw(int buffer[], int time) {
		draw(buffer,time,null,null);
	}

	public final void draw(int buffer[], int time, int image[], int map[]) {
		int offsetX = 0;
		int offsetY = 0;
		int imagewidth = width;
		if (image==null) {
			image = this.image;
			imagewidth = this.imagewidth;
			offsetX = xvar+(int)(xvar*Math.sin((float)(5000+time)/900));
			offsetY = yvar+(int)(yvar*Math.cos((float)(2000+time)/600));
		}
		if (map==null) map = this.map;


		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int r, g, b;
				int pixel, diff;
				pixel = image[(y+offsetY)*imagewidth+x+offsetX];
				diff = map[y*(width+1)+x]-map[y*(width+1)+x+1]+map[y*(width+1)+x]-map[(y+1)*(width+1)+x];
				r = (pixel&0x00FF0000)>>16;
				g = (pixel&0x0000FF00)>>8;
				b = (pixel&0x000000FF)>>0;
				r += diff;
				g += diff;
				b += diff;
				if (r<0) r = 0;
				else if (r>255) r = 255;
				if (g<0) g = 0;
				else if (g>255) g = 255;
				if (b<0) b = 0;
				else if (b>255) b = 255;
				buffer[pos++] = r<<16 | g<<8 | b;
			}
		}
	}

}


final class Distort extends Effect {

	public Distort(int w, int h) {
		super(w,h);
	}

	public final void draw(int buffer[], int time) {
	}

	public final void draw(int buffer[], int time, int image[], boolean bilinear) {
		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				float x2 = (float) (x + 5*Math.sin((float)y/70+(float)(25*x+time)/650));
				float y2 = (float) (y + 5*Math.cos((float)x/85+(float)(30*y+time)/900));
				if (x2<0) x2 = 0;
				if (y2<0) y2 = 0;
				if (x2>=width-1) x2 = width-2;
				if (y2>=height-1) y2 = height-2;
				int ix = (int) x2;
				int iy = (int) y2;
				float fx = x2 - ix;
				float fy = y2 - iy;

				int r1, r2, r3, r4, g1, g2, g3, g4, b1, b2, b3, b4;
				int pixel1, pixel2, pixel3, pixel4;


				pixel1 = image[iy*width+ix];
				r1 = (pixel1&0x00FF0000)>>16;
				g1 = (pixel1&0x0000FF00)>>8;
				b1 = (pixel1&0x000000FF)>>0;

				if (bilinear) {
					pixel2 = image[iy*width+ix+1];
					r2 = (pixel2&0x00FF0000)>>16;
					g2 = (pixel2&0x0000FF00)>>8;
					b2 = (pixel2&0x000000FF)>>0;
					pixel3 = image[(iy+1)*width+ix];
					r3 = (pixel3&0x00FF0000)>>16;
					g3 = (pixel3&0x0000FF00)>>8;
					b3 = (pixel3&0x000000FF)>>0;
					pixel4 = image[(iy+1)*width+ix+1];
					r4 = (pixel4&0x00FF0000)>>16;
					g4 = (pixel4&0x0000FF00)>>8;
						b4 = (pixel4&0x000000FF)>>0;

					int r, g, b;
					r = (int) (r1*(1-fx)*(1-fy) + r2*fx*(1-fy) + r3*(1-fx)*fy + r4*fx*fy);
					g = (int) (g1*(1-fx)*(1-fy) + g2*fx*(1-fy) + g3*(1-fx)*fy + g4*fx*fy);
					b = (int) (b1*(1-fx)*(1-fy) + b2*fx*(1-fy) + b3*(1-fx)*fy + b4*fx*fy);

					buffer[pos++] = r<<16 | g<<8 | b;
				} else {
					buffer[pos++] = pixel1;
				}

			}
		}
	}

}


final class BumpMap extends Effect {

	public BumpMap(int w, int h, Image c, Image b) {
		super(w,h);
		colorMapImage = c;
		bumpMapImage = b;
	}

	public final void init() {

		if (colorMapImage!=null)
			colorMap = Caffeine.makeArray(colorMapImage);
		if (bumpMapImage!=null) {
			bumpMap = Caffeine.makeArray(bumpMapImage);
			Caffeine.grayScale(bumpMap);
		}

		light = new short[256*256];

		int pos = 0;
		for (int y = 0; y < 256; y++) {
			for (int x = 0; x < 256; x++) {
				int dist = (int) Math.sqrt((128-x)*(128-x) + (128-y)*(128-y));
				if (dist > 127)
					light[pos++] = 0;
				else
					light[pos++] = (short) (255-dist*2);
			}
		}

	}

	private Image colorMapImage, bumpMapImage;
	private int colorMap[];
	private int bumpMap[];
	private short light[];

	public final void draw(int buffer[], int time) {
		draw(buffer,time,null,null);
	}

	public final void draw(int buffer[], int time, int colorMap[], int bumpMap[]) {
		if (colorMap==null) colorMap = this.colorMap;
		if (bumpMap==null) bumpMap = this.bumpMap;

		int light1X = width/2+(int)((width/2-15)*Math.sin((float)(1300+time)/600));
		int light1Y = height/2+(int)((height/2-15)*Math.cos((float)(900+time)/450));
		int zoom1 = (int) (295+75*Math.sin((float)time/800));
		int light2X = width/2+(int)((width/2-25)*Math.sin((float)(500+time)/750));
		int light2Y = height/2+(int)((height/2-25)*Math.cos((float)(1200+time)/600));
		int zoom2 = (int) (350+110*Math.sin((float)time/2100));

		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int intensity = 0, posX, posY;
				int diffX = bumpMap[y*(width+1)+x] - bumpMap[y*(width+1)+x+1];
				int diffY = bumpMap[y*(width+1)+x] - bumpMap[(y+1)*(width+1)+x];
				posX = 256*(x-light1X+zoom1/2)/zoom1 + diffX;
				posY = 256*(y-light1Y+zoom1/2)/zoom1 + diffY;
				if ((posX>=0) && (posX<256) && (posY>=0) && (posY<256))
					intensity += light[(posY<<8)+posX];
				posX = 256*(x-light2X+zoom2/2)/zoom2 + diffX;
				posY = 256*(y-light2Y+zoom2/2)/zoom2 + diffY;
				if ((posX>=0) && (posX<256) && (posY>=0) && (posY<256))
					intensity += light[(posY<<8)+posX];
				if (intensity>256) intensity = 256;
				buffer[pos] = shade(colorMap[pos],intensity);
				pos++;
			}
		}
	}

	private final static int shade(int pixel, int intensity) {
		int r, g, b;
		r = (pixel&0x00FF0000)>>16;
		g = (pixel&0x0000FF00)>>8;
		b = (pixel&0x000000FF)>>0;
		// /256
		r = (intensity*r)>>8;
		g = (intensity*g)>>8;
		b = (intensity*b)>>8;
		return r<<16 | g<<8 | b;
	}

}


final class Filter extends Effect {

	public Filter(int w, int h) {
		super(w,h);
	}

	public final void draw(int buffer[], int time) {
	}

	public final void draw(int buffer[], int image[], float filter[], float divisor, int offset, int num) {

/*		float filter[] = {
			1, 0, 0, 0, 1,
			0, 0, 0, 0, 0,
			0, 0, -1, 0, 0,
			0, 0, 0, 0, 0,
			1, 0, 0, 0, 1
		};
		float divisor = 3;
		float filter[] = {
			1, 1, 1,
			1, 1, 1,
			1, 1, 1
		};
		float divisor = 9;
		int offset = 0;
*/

		int buffer1[], buffer2[], tmp[];

		if (num==1) {
			applyFilter(buffer,image,filter,divisor,offset);
		} else {
			buffer1 = buffer;
			buffer2 = image;

			for (int i = 0; i < num; i++) {
				applyFilter(buffer1,buffer2,filter,divisor,offset);
				tmp = buffer1;
				buffer1 = buffer2;
				buffer2 = tmp;
			}

			if (num%2==0)
				System.arraycopy(buffer2,0,buffer,0,buffer.length);

		}


	}

	private final void applyFilter(int out[], int in[], float matrix[], float divisor, int offset) {
		int matrixSize = (int) Math.sqrt(matrix.length);
		if (matrixSize%2!=1) return;

		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				float rSum, gSum, bSum;
				int r, g, b;
				rSum = gSum = bSum = offset;
				int startX2 = x - (matrixSize>>1);
				int startY2 = y - (matrixSize>>1);
				int stopX2 = x + (matrixSize>>1);
				int stopY2 = y + (matrixSize>>1);
				if (startX2 < 0) startX2 = 0;
				if (startY2 < 0) startY2 = 0;
				int pos2 = 0;
				for (int y2 = startY2; y2 <= stopY2; y2++) {
					if (y2 >= height) break;
					for (int x2 = startX2; x2 <= stopX2; x2++) {
						if (x2 >= width) break;
						int rgb;
						rgb = in[y2*width+x2];
						r = (rgb&0x00FF0000)>>16;
						g = (rgb&0x0000FF00)>>8;
						b = (rgb&0x000000FF)>>0;
						rSum += matrix[pos2] * r;
						gSum += matrix[pos2] * g;
						bSum += matrix[pos2] * b;
						pos2++;
					}
				}

				r = (int)(rSum/divisor);
				g = (int)(gSum/divisor);
				b = (int)(bSum/divisor);

				if (r<0) r = 0;
				else if (r>255) r = 255;
				if (g<0) g = 0;
				else if (g>255) g = 255;
				if (b<0) b = 0;
				else if (b>255) b = 255;

				out[pos++] = r<<16 | g<<8 | b;
			}
		}
	}

}


final class Cube extends Effect {

	public Cube(int w, int h, int c) {
		super(w,h);
		color = c;
	}

	public final void init() {
		v = new Vertex[8];
		v[0] = new Vertex(1,-1,2);
		v[1] = new Vertex(-1,-1,2);
		v[2] = new Vertex(-1,1,2);
		v[3] = new Vertex(1,1,2);
		v[4] = new Vertex(1,-1,4);
		v[5] = new Vertex(-1,-1,4);
		v[6] = new Vertex(-1,1,4);
		v[7] = new Vertex(1,1,4);

		rotateVertex = new Vertex(0,0,3);

		lasttime = 0;
	}

	private final class Vertex {
		public float x;
		public float y;
		public float z;

		public Vertex(float x, float y, float z) {
			this.x = x;
			this.y = y;
			this.z = z;
		}
	}

	private Vertex v[];
	private Vertex rotateVertex;
	private int lasttime;
	private int color;

	public final void setColor(int c) {
		color = c;
	}

	public final void draw(int buffer[], int time) {
		draw(buffer,time,true);
	}

	public final void draw(int buffer[], int time, boolean clear) {
		int difftime = time - lasttime;
		lasttime = time;

		if (clear) Caffeine.arraySet(buffer,0);

		float zAngle = (float)difftime/900;
		float yAngle = (float)difftime/1200;
		float xAngle = (float)difftime/1300;

		for (int i = 0; i < v.length; i++) {
			rotateZ(v[i],zAngle,rotateVertex);
//			rotateZ(v[i],(float)(5*Math.PI/180),rotateVertex);
			rotateY(v[i],yAngle,rotateVertex);
			rotateX(v[i],xAngle,rotateVertex);
		}

		drawVertexLine(v[0],v[1],buffer);
		drawVertexLine(v[1],v[2],buffer);
		drawVertexLine(v[2],v[3],buffer);
		drawVertexLine(v[3],v[0],buffer);
		drawVertexLine(v[4],v[5],buffer);
		drawVertexLine(v[5],v[6],buffer);
		drawVertexLine(v[6],v[7],buffer);
		drawVertexLine(v[7],v[4],buffer);
		drawVertexLine(v[0],v[4],buffer);
		drawVertexLine(v[1],v[5],buffer);
		drawVertexLine(v[2],v[6],buffer);
		drawVertexLine(v[3],v[7],buffer);
	}


	private final void rotateZ(Vertex v, float angle, Vertex c) {
		float radius = (float) Math.sqrt((v.x-c.x)*(v.x-c.x)+(v.y-c.y)*(v.y-c.y));
		float startAngle = (float) Math.atan2(v.y-c.y,v.x-c.x);
		v.x = c.x + (float) (radius*Math.cos(startAngle+angle));
		v.y = c.y + (float) (radius*Math.sin(startAngle+angle));
	}

	private final void rotateY(Vertex v, float angle, Vertex c) {
		float radius = (float) Math.sqrt((v.x-c.x)*(v.x-c.x)+(v.z-c.z)*(v.z-c.z));
		float startAngle = (float) Math.atan2(v.z-c.z,v.x-c.x);
		v.x = c.x + (float) (radius*Math.cos(startAngle+angle));
		v.z = c.z + (float) (radius*Math.sin(startAngle+angle));
	}

	private final void rotateX(Vertex v, float angle, Vertex c) {
		float radius = (float) Math.sqrt((v.y-c.y)*(v.y-c.y)+(v.z-c.z)*(v.z-c.z));
		float startAngle = (float) Math.atan2(v.z-c.z,v.y-c.y);
		v.y = c.y + (float) (radius*Math.cos(startAngle+angle));
		v.z = c.z + (float) (radius*Math.sin(startAngle+angle));
	}

	private final void drawVertexLine(Vertex v1, Vertex v2, int buffer[]) {
		Point p1, p2;

		p1 = getScreenCoord(v1);
		p2 = getScreenCoord(v2);
		drawLine(p1.x,p1.y,p2.x,p2.y,color,buffer);
		drawLine(p1.x+1,p1.y,p2.x+1,p2.y,color,buffer);
		drawLine(p1.x,p1.y+1,p2.x,p2.y+1,color,buffer);
		drawLine(p1.x+1,p1.y+1,p2.x+1,p2.y+1,color,buffer);
	}

	private final Point getScreenCoord(Vertex v) {
		final float screenZ = (float) 1.5;
		final float screenY1 = (float) -1.2;
		final float screenY2 = (float) 1.2;
		final float screenX1 = (float) -1.6;
		final float screenX2 = (float) 1.6;
		int x=0,y;

		float yCoord = screenZ * v.y / v.z;
		float xCoord = screenZ * v.x / v.z;

		if (yCoord<screenY1 || yCoord>screenY2) return null;
		if (xCoord<screenX1 || xCoord>screenX2) return null;

		y = (int) (height*(screenY2-yCoord)/(screenY2-screenY1));
		x = (int) (width*(xCoord-screenX1)/(screenX2-screenX1));

		return new Point(x,y);
	}

	private final void drawLine(int x1, int y1, int x2, int y2, int color, int buffer[]) {
		int deltaX = x2 - x1;
		int deltaY = y2 - y1;
		int deltaXAbs = (deltaX<0) ? -deltaX : deltaX;
		int deltaYAbs = (deltaY<0) ? -deltaY : deltaY;
		int signDeltaX = (deltaX<0) ? -1 : (deltaX>0) ? 1 : 0;
		int signDeltaY = (deltaY<0) ? -1 : (deltaY>0) ? 1 : 0;
		int x = deltaYAbs>>1;
		int y = deltaXAbs>>1;
		int currentX = x1;
		int currentY = y1;

		buffer[currentY*width+currentX] = color;

		if (deltaXAbs >= deltaYAbs) {
			for (int i = 0; i < deltaXAbs; i++) {
				y += deltaYAbs;
				if (y >= deltaXAbs) {
					y -= deltaXAbs;
					currentY += signDeltaY;
				}
				currentX += signDeltaX;
				buffer[currentY*width+currentX] = color;
			}
		} else {
			for (int i = 0; i < deltaYAbs; i++) {
				x += deltaXAbs;
				if (x >= deltaYAbs) {
					x -= deltaYAbs;
					currentX += signDeltaX;
				}
				currentY += signDeltaY;
				buffer[currentY*width+currentX] = color;
			}
		}
	}


}


final class Fire extends Effect {

	public Fire(int w, int h) {
		super(w,h);
	}

	public final void init() {
		max = 192;

		int firetablelength = 512;
		xtable = new short[firetablelength];
		ytable = new short[firetablelength];

		for (int i = 0; i < firetablelength; i++) {
//			xtable[i] = (int) (65*Math.sin(2*Math.PI*i/firetablelength));
//			ytable[i] = (int) (65*Math.cos(2*Math.PI*i/firetablelength));
			xtable[i] = (short) (2*65*Math.sin(2*Math.PI*i/firetablelength));
			ytable[i] = (short) (65*Math.sin(4*Math.PI*i/firetablelength));
		}

		buffer1 = new int[width*height];
		buffer2 = new int[width*height];

		palette = createPalette().getPaletteArray();
	}

	private short xtable[];
	private short ytable[];
	private int buffer1[], buffer2[];
	private int palette[];
	private int max;

	private final Palette createPalette() {
		Palette p = new Palette(256);

/*		for (int i = 0; i < 64; i++)
			p.setRGB(i,i<<2,0,0);
*/		for (int i = 0; i < 32; i++)
			p.setRGB(i,0,0,i<<1);
		for (int i = 32; i < 64; i++)
			p.setRGB(i,(i-32)<<3,0,(63-i)<<1);
		for (int i = 64; i < 128; i++)
			p.setRGB(i,255,(i-64)<<2,0);
		for (int i = 128; i < 192; i++)
			p.setRGB(i,255,255,(i-128)<<2);
		for (int i = 192; i < 256; i++)
			p.setRGB(i,255,255,255);

		return p;
	}

	public final void reset() {
		Caffeine.arraySet(buffer2,0);
	}

	public final void draw(int buffer[], int time) {
		draw(buffer,time,true,1);
	}

	public final void draw(int buffer[], int time, boolean lightGround, int lightExtra) {
		int n;
		switch (lightExtra) {
			case 0:
				break;

			case 1:
				int startrow = (height/2)*width;
				int offset = startrow + 1 + (int)(time/7)%(width-2);
				n = 20+Caffeine.rand(20);
				for (int j = 0; j < n; j++) {
					int pos = Caffeine.rand(50);
					pos += offset;
					if (pos >= startrow+width-1) pos -= (width-2);
					buffer2[pos] = max;
				}
				break;

			case 2:
				int middlex = width/2;
				int middley = height/2;

				n = 40+Caffeine.rand(70);
				for (int j = 0; j < n; j++) {
					int k = Caffeine.rand(75);
					int l = Caffeine.rand(4);
					int diffx = xtable[(int)((time-1000)/8+k)&0x01FF];
					int diffy = ytable[(int)((time-1000)/8+k)&0x01FF];
					if (l >= 2) {
						l -= 2;
						diffx++;
					}
					if (l >= 1) diffy++;

					buffer2[(middley+diffy)*width+middlex+diffx] = max;
				}
				break;
		}

		if (lightGround) {
//			n = 300+Caffeine.rand(1000);
			n = 50+Caffeine.rand(350);
			for (int j = 0; j < n; j++)
				buffer2[(height-7+Caffeine.rand(5))*width+1+Caffeine.rand(width-2)] = max;
		}

		for (int y = 0; y < (height-2); y++) {
			for (int x = 1; x < (width-1); x++) {
				int offset = y*width+x;
				buffer1[offset] = (buffer2[offset-1] + buffer2[offset+1] + buffer2[offset+width-1] + buffer2[offset+width] + buffer2[offset+width+1] + buffer2[offset+width+width-1] + buffer2[offset+width+width] + buffer2[offset+width+width+1])>>3;
				if (buffer1[offset]>0) buffer1[offset]--;
			}
		}

		for (int i = 0; i < buffer.length; i++)
			buffer[i] = palette[buffer1[i]];


		int tmp[] = buffer1;
		buffer1 = buffer2;
		buffer2 = tmp;

	}


}


final class Tunnel extends Effect {

	public Tunnel(int w, int h) {
		super(w,h);
	}

	public final void init() {
		int middleX, middleY;
		middleX = width/2;
		middleY = height/2;

		depthtable = new int[width*height];
		float borderdepth = (float) Math.sqrt(Math.sqrt(middleX*middleX+middleY*middleY))+1;
		int pos = 0;
		for (int y = 0; y < height; y++)
			for (int x = 0; x < width; x++)
				depthtable[pos++] = (int) ((1<<16)*(borderdepth - Math.sqrt(Math.sqrt((x-middleX)*(x-middleX)+(y-middleY)*(y-middleY)))));
		xtable = new int[width*height];
		pos = 0;
		for (int y = 0; y < height; y++)
			for (int x = 0; x < width; x++)
				xtable[pos++] = (int) ((1<<16)*128*Math.atan2((y-middleY),(x-middleX))/Math.PI);

		Palette p = new Palette(4);
		p.setRGB(0,0,0,0);
		p.setRGB(1,0,255,0);
		p.setRGB(2,0,0,255);
		p.setRGB(3,255,0,0);
		palette = p.getPaletteArray();
	}

	private int depthtable[];
	private int xtable[];
	private int palette[];

	private final byte texture[] = {
/*		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
		1,0,2,2,0,0,0,0,0,0,0,0,0,2,2,0,
		1,0,2,2,2,0,0,0,0,0,0,0,2,2,2,0,
		1,0,0,2,2,2,0,0,0,0,0,2,2,2,0,0,
		1,0,0,0,2,2,2,0,0,0,2,2,2,0,0,0,
		1,0,0,0,0,2,2,2,0,2,2,2,0,0,0,0,
		1,0,0,0,0,0,2,2,2,2,2,0,0,0,0,0,
		1,0,0,0,0,0,0,2,2,2,0,0,0,0,0,0,
		1,0,0,0,0,0,2,2,2,2,2,0,0,0,0,0,
		1,0,0,0,0,2,2,2,0,2,2,2,0,0,0,0,
		1,0,0,0,2,2,2,0,0,0,2,2,2,0,0,0,
		1,0,0,2,2,2,0,0,0,0,0,2,2,2,0,0,
		1,0,2,2,2,0,0,0,0,0,0,0,2,2,2,0,
		1,0,2,2,0,0,0,0,0,0,0,0,0,2,2,0,
		1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
*/
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,
		1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,
		1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,
		1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,
		1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,
		1,1,0,0,0,0,0,2,2,2,0,0,0,0,0,1,1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,
		1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,
		1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,
		1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,
		1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,
		1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,
		1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,
		1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,
		1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,
		1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,
		1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,1,1,0,0,0,0,0,2,2,2,0,0,0,0,0,1,
		1,1,0,3,3,3,0,0,0,0,0,3,3,3,0,1,1,1,0,0,0,0,2,2,2,2,2,0,0,0,0,1,
		1,1,0,0,3,3,3,0,0,0,3,3,3,0,0,1,1,1,0,0,0,2,2,2,0,2,2,2,0,0,0,1,
		1,1,0,0,0,3,3,3,3,3,3,3,0,0,0,1,1,1,0,0,2,2,2,0,0,0,2,2,2,0,0,1,
		1,1,0,0,0,0,3,3,3,3,3,0,0,0,0,1,1,1,0,2,2,2,0,0,0,0,0,2,2,2,0,1,
		1,1,0,0,0,0,0,3,3,3,0,0,0,0,0,1,1,1,0,2,2,0,0,0,0,0,0,0,2,2,0,1,
		1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,
		1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1
	};

	public final void draw(int buffer[], int time) {

		int dy = (int) (time<<12);
		int dx = (int) ((1<<16)*30*Math.sin((float)time/3000));

		int pos = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				buffer[pos] = palette[texture[(((10*depthtable[pos]+dy)>>11)&0x03E0)+(((xtable[pos]+dx)>>16)&0x1F)]];
				pos++;
			}
		}

	}

}


final class FadeText extends Effect {

	public FadeText(int w, int h) {
		super(w,h);
	}

	public final void init() {
		overlay = new int[width*height];
	}

	private int font[];
	private String fontChars;
	private boolean lowerCase;
	private int overlay[];
	private int charWidth, charHeight;
	private int fontImageWidth;
	private String text;
	private int charTime;
	private int startTime;
	private int lastTime;
	private int currentChar;
	private int x, y, leftMargin;
	private boolean done;

	public final void setFont(int f[], String chars, boolean lc, int cw, int ch, int w) {
		font = f;
		fontChars = chars;
		lowerCase = lc;
		charWidth = cw;
		charHeight = ch;
		fontImageWidth = w;
	}

	public final void setText(String t, int time, int x, int y) {
		text = t;
		this.x = x;
		this.y = y;
		leftMargin = x;

		int numChars = 0;
		for (int i = 0; i < t.length(); i++) {
			char c = t.charAt(i);
			if (c != ' ' && c != '\n') numChars++;
		}

		charTime = time/numChars;

		startTime = -1;
		lastTime = 0;
		currentChar = 0;
		done = false;

		Caffeine.arraySet(overlay,0);
	}

	public final void draw(int buffer[], int time) {

		if (startTime < 0) startTime = time;
		time -= startTime;

		if (!done) {
			while ((time - lastTime) > charTime) {
				lastTime += charTime;
				drawChar(255);
				nextChar();
				if (done) break;
			}

			if (!done) drawChar(255*(time-lastTime)/charTime);
		}


		drawOverlay(buffer);

	}

	private final void nextChar() {
		while (true) {
			currentChar++;

			if (currentChar>=text.length()) {
				done = true;
				break;
			}

			char c = text.charAt(currentChar);
			if (c == ' ') {
				x += charWidth;
			} else if (c == '\n') {
				x = leftMargin - charWidth;
				y += charHeight;
			} else {
				x += charWidth;
				break;
			}
		}
	}

	private final int getIndex(char c) {
		if (!lowerCase) c = Character.toUpperCase(c);
		return fontChars.indexOf(c);
	}

	private final void drawChar(int alpha) {
		int index = getIndex(text.charAt(currentChar));

		int fontX = charWidth*(index%fontImageWidth);
		int fontY = charHeight*(index/fontImageWidth);
		int fontStart = fontY*fontImageWidth*charWidth + fontX;
		int overlayStart = this.y*width+this.x;

		for (int y = 0; y < charHeight; y++) {
			for (int x = 0; x < charWidth; x++) {
				int fontColor = font[fontStart+x]&0x00FFFFFF;
				if (fontColor > 0)
					overlay[overlayStart+x] = alpha<<24 | fontColor;
				else
					overlay[overlayStart+x] = 0x00000000;
			}
			fontStart += fontImageWidth*charWidth;
			overlayStart += width;
		}
	}

	private final void drawOverlay(int buffer[]) {
		for (int i = 0; i < overlay.length; i++) {
			int alpha = ((overlay[i]&0xFF000000)>>24)&0xFF;
			if (alpha==0)
				continue;
			else if (alpha==255)
				buffer[i] = overlay[i]&0x00FFFFFF;
			else {
				int r1, g1, b1;
				int r2, g2, b2;
				r1 = (buffer[i]&0x00FF0000)>>16;
				g1 = (buffer[i]&0x0000FF00)>>8;
				b1 = (buffer[i]&0x000000FF);
				r2 = (overlay[i]&0x00FF0000)>>16;
				g2 = (overlay[i]&0x0000FF00)>>8;
				b2 = (overlay[i]&0x000000FF);
				int r, g, b;
				// /255
				r = r1 + (alpha*(r2-r1)>>8);
				g = g1 + (alpha*(g2-g1)>>8);
				b = b1 + (alpha*(b2-b1)>>8);
				buffer[i] = r<<16 | g<<8 | b;
			}
		}
	}

}


final class ShowText extends Effect {

	public ShowText(int w, int h) {
		super(w,h);
	}

	public final void init() {
	}

	private int font[];
	private String fontChars;
	private boolean lowerCase;
	private int charWidth, charHeight;
	private int fontImageWidth;
	private boolean plain;
	private int color;

	private int texts[][];
	private int bufferWidth, bufferHeight;

	private int textTime;

	private int startTime;

	public final void setFont(int f[], String chars, boolean lc, int cw, int ch, int w, boolean p, int c) {
		font = f;
		fontChars = chars;
		lowerCase = lc;
		charWidth = cw;
		charHeight = ch;
		fontImageWidth = w;
		plain = p;
		color = c;
	}

	public final void setText(String t[], int time) {
		int maxChars = 0;
		int maxRows = 0;
		for (int i = 0; i < t.length; i++) {
			t[i] = t[i].toUpperCase();
			String s = t[i];
			int chars = 0, rows = 1;
			for (int j = 0; j < s.length(); j++) {
				if (s.charAt(j) == '\n') {
					rows++;
					if (chars > maxChars) maxChars = chars;
					chars = 0;
				} else {
					chars++;
				}
			}
			if (chars > maxChars) maxChars = chars;
			if (rows > maxRows) maxRows = rows;
		}

		bufferWidth = maxChars*charWidth;
		bufferHeight = maxRows*charHeight;

		texts = new int[t.length][];
		for (int i = 0; i < t.length; i++) {
			texts[i] = new int[bufferWidth*bufferHeight];
			drawString(texts[i],t[i]);
		}


		textTime = time/t.length;

		startTime = -1;
	}

	private final void drawString(int buffer[], String s) {
		int rows = 1;
		for (int i = 0; i < s.length(); i++)
			if (s.charAt(i) == '\n')
				rows++;

		int y = (bufferHeight-rows*charHeight)/2;

		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < s.length(); i++) {
			char c = s.charAt(i);
			if (c == '\n') {
				drawRow(buffer,sb.toString(),y);
				sb = new StringBuffer();
				y += charHeight;
			} else {
				sb.append(c);
			}
		}
		drawRow(buffer,sb.toString(),y);
	}

	private final void drawRow(int buffer[], String s, int y) {
		int x = (bufferWidth-s.length()*charWidth)/2;

		for (int i = 0; i < s.length(); i++) {
			drawChar(buffer,s.charAt(i),x,y);
			x += charWidth;
		}
	}

	private final int getIndex(char c) {
		if (!lowerCase) c = Character.toUpperCase(c);
		return fontChars.indexOf(c);
	}

	private final void drawChar(int buffer[], char c, int bufferX, int bufferY) {
		int index = getIndex(c);

		int fontX = charWidth*(index%fontImageWidth);
		int fontY = charHeight*(index/fontImageWidth);
		int fontStart = fontY*fontImageWidth*charWidth + fontX;

		int bufferStart = bufferY*bufferWidth+bufferX;

		for (int y = 0; y < charHeight; y++) {
			for (int x = 0; x < charWidth; x++) {
				int fontColor = font[fontStart+x]&0x00FFFFFF;
				if (fontColor > 0)
					buffer[bufferStart+x] = 0xFF000000 | fontColor;
				else
					buffer[bufferStart+x] = 0x00000000;
			}
			fontStart += fontImageWidth*charWidth;
			bufferStart += bufferWidth;
		}
	}

	public final void draw(int buffer[], int time) {
		if (startTime < 0) startTime = time;
		time -= startTime;

		int textNr = time/textTime;
		int textPos = time%textTime;

		if (textNr>=texts.length) return;

		int x = (width-bufferWidth)/2;
		int y = 0;

		if (textPos < 20*textTime/100) {
			// Fall
			y = (int) ((height-bufferHeight)/2 - ((height-bufferHeight)/2 + bufferHeight)*Math.cos((Math.PI/2)*textPos/(20*textTime/100)));
		} else if (textPos < 30*textTime/100) {
			// Bounce
			y = (int) ((height-bufferHeight)/2 - (0.20*(height-bufferHeight)/2)*Math.sin(Math.PI*(textPos-20*textTime/100)/(10*textTime/100)));
		} else if (textPos < 75*textTime/100) {
			y = (height-bufferHeight)/2;
		} else {
			// Fall
			y = (int) (height - (height-(height-bufferHeight)/2)*Math.cos((Math.PI/2)*(textPos-75*textTime/100)/(25*textTime/100)));
		}

		drawOverlay(buffer,texts[textNr],x,y);
	}

	private final void drawOverlay(int buffer[], int overlay[], int x, int y) {

		int overlayXStart = 0, overlayYStart = 0;
		int bufferXStart = x, bufferYStart = y;

		if (x < 0) {
			overlayXStart = -x;
			bufferXStart = 0;
		}
		if (y < 0) {
			overlayYStart = -y;
			bufferYStart = 0;
		}

		int by = bufferYStart;
		for (int oy = overlayYStart; oy < bufferHeight; oy++) {
			if (by>=height) break;
			int bx = bufferXStart;
			for (int ox = overlayXStart; ox < bufferWidth; ox++) {
				if (bx>=width) break;
				int c = overlay[oy*bufferWidth+ox]&0x00FFFFFF;
				if (c>0)
					if (plain)
						buffer[by*width+bx] = color;
					else
						buffer[by*width+bx] = c;
				bx++;
			}
			by++;
		}
	}

}


final class Invert extends Effect {

	public Invert(int w, int h, DataExchangeEffect dee) {
		super(w,h);
		exchange = dee;
	}

	public final void init() {
		if (exchange==null) {
			sintable = new short[512];
			for (int i = 0; i < 512; i++)
				sintable[i] = (short) (128+127*Math.sin(2*Math.PI*i/512));

		} else {
			Object data[] = exchange.getData();
			sintable = (short[]) data[0];
		}
	}

	private DataExchangeEffect exchange;
	private short sintable[];

	public final void draw(int buffer[], int time) {
		draw(buffer,time,buffer);
	}

	public final void draw(int buffer[], int time, int in[]) {

		int pos = 0;
		for (int y = 0; y < height; y++) {
			int xPos = 100 + sintable[(y-time/5)&0x01FF]/3 + (25+sintable[(y*2-time/7)&0x01FF]/2)*sintable[(y*10+2*time/3)&0x01FF]/800;

/*			for (int x = 0; x < width; x++) {
				if (x<xPos)
					buffer[pos] = ~in[pos];
				else
					buffer[pos] = in[pos];
				pos++;
			}
*/
			for (int x = 0; x < xPos; x++) {
				buffer[pos] = ~in[pos];
				pos++;
			}
			xPos = width-xPos;
			for (int x = 0; x < xPos; x++) {
				buffer[pos] = in[pos];
				pos++;
			}

		}
	}

}

final class Checkers extends Effect {

	public Checkers(int w, int h) {
		super(w,h);
	}

	public final void init() {
		squareWidth = 40;
		squareHeight = 40;
	}

	private int squareWidth;
	private int squareHeight;


	public final void draw(int buffer[]) {
		draw(buffer,buffer);
	}

	public final void draw(int buffer[], int time) {
		draw(buffer,buffer);
	}

	public final void draw(int buffer[], int time, int in[]) {
		draw(buffer,in);
	}

	public final void draw(int buffer[], int in[]) {

		int pos = 0;
/*		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				boolean firstX = (x%(squareWidth*2)<squareWidth);
				boolean firstY = (y%(squareHeight*2)<squareHeight);
				if ((firstX && !firstY) || (!firstX && firstY)) buffer[pos] = ~in[pos];
				else buffer[pos] = in[pos];
				pos++;
			}
		}
*/
		boolean invert = false;
		for (int y = 0; y < height; y++) {
			if (y%(squareHeight*2)<squareHeight) invert = false;
			else invert = true;

			for (int x = 0; x < width; x++) {
				if (x%squareHeight==0) invert = invert ? false : true;

				if (invert) buffer[pos] = ~in[pos];
				else buffer[pos] = in[pos];
				pos++;
			}
		}

	}

}

final class Mini extends Effect {

	public Mini(int w, int h) {
		super(w,h);
	}

	public final void init() {
		miniBuffer = new int[(width/4)*(height/4)];
	}

	private int miniBuffer[];

	public final void draw(int buffer[], int time) {
		draw(buffer);
	}

	public final void draw(int buffer[]) {
		int pos = 0;
		for (int y = 0; y < height; y += 4) {
			int row = y*width;
			for (int x = 0; x < width; x += 4)
				miniBuffer[pos++] = buffer[row+x];
		}

		for (int y = 0; y < height/4; y++)
			System.arraycopy(miniBuffer,y*width/4,buffer,y*width,width/4);
	}

}

final class Growing extends Effect {

	public Growing(int w, int h) {
		super(w,h);
	}

	public final void init() {
		distTable = new int[width*height];

		Point points[] = new Point[3];
		int extra[] = new int[3];

		for (int i = 0; i < points.length; i++) {
			points[i] = new Point();
//			points[i].x = Caffeine.rand(width);
//			points[i].y = Caffeine.rand(height);
		}

		extra[0] = 0;
		extra[1] = 20;
		extra[2] = 55;

		points[0].x = 135;
		points[0].y = 150;
		points[1].x = 250;
		points[1].y = 50;
		points[2].x = 230;
		points[2].y = 200;

		int pos = 0;
		maxDist = 0;
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				int min = (int)((1<<8)*(Math.sqrt((points[0].x-x)*(points[0].x-x)+(points[0].y-y)*(points[0].y-y))+extra[0]));

				for (int i = 1; i < points.length; i++) {
					int d = (int)((1<<8)*(Math.sqrt((points[i].x-x)*(points[i].x-x)+(points[i].y-y)*(points[i].y-y))+extra[i]));
					if (d<min) min = d;
				}

				if (min>maxDist) maxDist = min;

				distTable[pos++] = min;
			}
		}
	}

	private int distTable[];
	private int maxDist;

	public final void draw(int buffer[], int time) {
	}

	public final void draw(int out[], int in1[], int in2[], int time, int duration) {
		int pos = maxDist*time/duration;
		for (int i = 0; i < out.length; i++) {
			if ((distTable[i]>>9)==(pos>>9)) out[i] = 0;
			else if (distTable[i]<pos) out[i] = in2[i];
			else out[i] = in1[i];
		}
	}

}




final class Palette {

	private int size;
	private byte red[];
	private byte green[];
	private byte blue[];

	public Palette(int s) {
		size = s;
		red = new byte[size];
		green = new byte[size];
		blue = new byte[size];
	}

	public final void setRGB(int index, int r, int g, int b) {
		red[index] = (byte) r;
		green[index] = (byte) g;
		blue[index] = (byte) b;
	}

	public final int getSize() {
		return size;
	}

	public final byte[] getRed() {
		return red;
	}

	public final byte[] getGreen() {
		return green;
	}

	public final byte[] getBlue() {
		return blue;
	}

	public final int[] getPaletteArray() {
		int array[] = new int[size];
		// No unsigned variables... *sigh*
		for (int i = 0; i < size; i++)
			array[i] = ((red[i]&0xFF)<<16) | ((green[i]&0xFF)<<8) | (blue[i]&0xFF);
		return array;
	}

}

final class Jar {
	public final static Image getImageFromJar(String filename) {
		InputStream is = (new Jar()).getClass().getResourceAsStream(filename);
		if (is==null) return null;
		byte data[];
		int length;

		try {
			ByteArrayOutputStream baos = new ByteArrayOutputStream();
			byte buffer[] = new byte[10240];
			while ((length = is.read(buffer)) > 0)
				baos.write(buffer,0,length);
			is.close();
			data = baos.toByteArray();
		} catch (IOException ioe) {
			System.err.println(ioe);
			return null;
		}

		return Toolkit.getDefaultToolkit().createImage(data);
	}
}


