
import com.ms.directX.*;
import java.io.*;


final class DirectSoundDevice implements AudioDevice, Runnable {

	private DirectSound ds;
	private int hWnd;
	private java.awt.Component component;
	private DirectSoundBuffer soundBuffer;
	private WaveFormatEx format;
	private DSCursors cursor;
	private Thread thread;
	private InputStream input;
	private byte byteBuffer[];

	public DirectSoundDevice(InputStream is) {
		this(is,null);
	}

	public DirectSoundDevice(InputStream is, java.awt.Component c) {
		input = is;
		component = c;

		ds = new DirectSound();
		getHWnd();
		if (hWnd!=0)
			ds.setCooperativeLevel(hWnd,DirectXConstants.DSSCL_PRIORITY);
		else
			ds.setCooperativeLevel(getComponent(),DirectXConstants.DSSCL_PRIORITY);

		format = new WaveFormatEx();
		format.bitsPerSample = 16;
		format.channels = 2;
		format.blockAlign = 4;
		format.samplesPerSec = 44100;
		format.avgBytesPerSec = 44100 * 4;
		format.formatTag = 1;
		format.extra = null;
		format.size = 0;

		DSBufferDesc bufferDesc = new DSBufferDesc();
		bufferDesc.flags = DirectXConstants.DSBCAPS_STICKYFOCUS;
		bufferDesc.bufferBytes = 10240;
		soundBuffer = ds.createSoundBuffer(bufferDesc,format);

		cursor = new DSCursors();

		byteBuffer = new byte[10240];
	}

	private final int getHWnd() {
		if (hWnd!=0) return hWnd;
		try {
			hWnd = com.ms.win32.User32.GetForegroundWindow();
		} catch (Exception e) {
			hWnd = 0;
		}
		if (hWnd!=0) return hWnd;
		try {
			hWnd = com.ms.win32.User32.GetDesktopWindow();
		} catch (Exception e) {
			hWnd = 0;
		}
		return hWnd;
	}

	public final java.awt.Component getComponent() {
		if (component!=null) return component;
		component = new java.awt.Frame();
		component.setSize(0,0);
		component.setVisible(true);
		component.validate();
		return component;
	}

	public final int getSampleRate() {
		return 44100;
	}

	public final boolean isStereo() {
		return true;
	}

	public final boolean is16Bit() {
		return true;
	}

	public final void start() {
		if (thread==null) {
			thread = new Thread(this);
			thread.setPriority(Thread.MAX_PRIORITY);
			thread.start();
		}
	}

	public final void stop() {
		thread = null;
	}


	public final void run() {
		soundBuffer.play(DirectXConstants.DSBPLAY_LOOPING);
		while (thread!=null) {
			for (int i = 0; i < 10; i++) {
				if (thread==null) break;
				try {
					input.read(byteBuffer,i*1024,1024);
				} catch (IOException ioe) {
				}
				while (true) {
					soundBuffer.getCurrentPosition(cursor);
					if (cursor.play < i*1024 || cursor.play > (i+1)*1024) break;
					try {
						Thread.sleep(5);
					} catch (InterruptedException ie) {
					}
				}
				soundBuffer.writeBuffer(i*1024,1024,byteBuffer,0);
			}
		}
		soundBuffer.stop();
	}


}


