
import java.applet.*;
import java.awt.*;
import java.awt.event.*;

public final class FullScreen extends Applet implements KeyListener {

	private Window window;
	private Applet applet;

	private int screenWidth, screenHeight;
	private int appletWidth, appletHeight;

	public final String[][] getParameterInfo() {
		String info[][] = {
			{"appletcode","String","Name of the applet to run"},
			{"appletwidth","int","Width of the applet to run"},
			{"appletheight","int","Height of the applet to run"}
		};
		return info;
	}

	public final void init() {
		String appletName = getParameter("appletcode");
		if (appletName==null) return;
		try {
			appletWidth = Integer.parseInt(getParameter("appletwidth"));
			appletHeight = Integer.parseInt(getParameter("appletheight"));
		} catch (NumberFormatException nfe) {
			return;
		} catch (NullPointerException npe) {
			return;
		}

		Class appletClass;
		try {
			appletClass = Class.forName(appletName);
		} catch (ClassNotFoundException cnfe) {
			System.err.println("Class "+appletName+" not found!");
			return;
		}

		Container c = this;
		do {
			try {
				c = c.getParent();
			} catch (NullPointerException npe) {
				c = new Frame();
				c.setBounds(0,0,0,0);
				c.setVisible(true);
				break;
			}
		} while (!(c instanceof Frame));

		window = new Window((Frame) c);
		screenWidth = window.getToolkit().getScreenSize().width;
		screenHeight = window.getToolkit().getScreenSize().height;
		window.setLocation(0,0);
		window.setSize(screenWidth,screenHeight);
		window.setBackground(Color.black);
		window.addKeyListener(this);

		try {
			applet = (Applet) appletClass.newInstance();
		} catch (InstantiationException ie) {
			return;
		} catch (IllegalAccessException iae) {
			return;
		}
		applet.addKeyListener(this);
		applet.setSize(appletWidth,appletHeight);
		window.setVisible(true);
		window.setLayout(new BorderLayout());
		window.add(applet);
		applet.setBounds((screenWidth-appletWidth)/2,(screenHeight-appletHeight)/2,appletWidth,appletHeight);
		applet.init();
	}

	public final void start() {
		window.setVisible(true);
		window.toFront();
		window.requestFocus();
		applet.setBounds((screenWidth-appletWidth)/2,(screenHeight-appletHeight)/2,appletWidth,appletHeight);
		applet.start();
	}

	public final void stop() {
		applet.stop();
		window.setVisible(false);
	}

	public final void destroy() {
		applet.destroy();
	}

	public final void paint() {
	}

	public final void keyPressed(KeyEvent ke) {
		if (ke.getKeyCode()==KeyEvent.VK_ESCAPE) stop();
	}
	public final void keyTyped(KeyEvent ke) {
	}
	public final void keyReleased(KeyEvent ke) {
	}


}

