
import java.io.*;
import javax.sound.sampled.*;

final class JavaSoundDevice implements AudioDevice, Runnable {

	private SourceDataLine line;
	private int bytesPerSec;
	private int bufferSize;
	private InputStream input;
	private Thread thread;

	public JavaSoundDevice(InputStream is) throws Exception {
		input = is;

		AudioFormat format = new AudioFormat(44100,16,2,true,false);
//		AudioFormat format = new AudioFormat(8000,8,1,true,false);

		DataLine.Info info = new DataLine.Info(SourceDataLine.class,format);

		if (!AudioSystem.isLineSupported(info))
			throw new Exception("No supported lines!");

		line = (SourceDataLine) AudioSystem.getLine(info);
		line.open(format);
//		line.open(format,4*44100*150/1000);

		bufferSize = line.getBufferSize();
		bytesPerSec = line.getFormat().getFrameSize()*(int)line.getFormat().getSampleRate();
	}

	public final int getSampleRate() {
		return (int) line.getFormat().getSampleRate();
	}

	public final boolean isStereo() {
		return line.getFormat().getChannels()==2;
	}

	public final boolean is16Bit() {
		return line.getFormat().getSampleSizeInBits()==16;
	}

	public final void start() {
		if (thread==null) {
			thread = new Thread(this);
			thread.start();
		}
	}

	public final void run() {
		line.start();
		byte data[] = new byte[512];
		while (thread!=null) {
			while (getDelay()>135) {
				try {
					Thread.sleep(5);
				} catch (InterruptedException ie) {
				}
			}
			try {
				input.read(data,0,data.length);
				line.write(data,0,data.length);
			} catch (IOException ioe) {
			}
		}
		line.drain();
		line.stop();
		line.close();
	}

	private final int getDelay() {
		return 1000*(bufferSize-line.available())/(bytesPerSec);
	}

	public final void stop() {
		thread = null;
	}

}


