
import java.io.*;

interface ModListener {
	public void channelHit();
	public void extraEffect(int effect, int parameter);
	public void newPattern(int order);
	public void newRow(int row, int pattern, int order, int length);
}


final class Mod extends InputStream {

	private final class Sample {
		public String name;
		public int length;
		public byte finetune;
		public byte volume;
		public int loopStart;
		public int loopLength;

		public byte data[];
	}

	private final class Note {
		public short note;
		public byte sample;
		public byte effect;
		public short parameter;
	}


	private String songName;
	private Sample samples[];
	private int songLength;
	private int channels;
	private byte orders[];
	private Note patterns[][];

	public final String getName() {
		return songName;
	}

	public final String getSampleInfo() {
		int n = samples.length/2;
		if (samples.length%2==1) n++;

		StringBuffer sb = new StringBuffer();
		for (int i = 0; i < n; i++) {
			if (i+1<10) sb.append(' ');
			sb.append(i+1);
			sb.append(' ');
			sb.append(samples[i].name);

			if (i+n<samples.length) {
				int k = samples[i].name.indexOf('\0');
				int m;
				if (k<0)
					m = 1;
				else
					m = 22-k+1;
				for (int j = 0; j < m; j++) sb.append(' ');
				if (i+n+1<10) sb.append(' ');
				sb.append(i+n+1);
				sb.append(' ');
				sb.append(samples[i+n].name);
			}

			sb.append('\n');
		}

		return sb.toString();
	}



	public Mod(InputStream is) throws Exception {
		this(is,null);
	}

	private void readFully(InputStream in, byte array[], int offset, int length) throws IOException {
		int pos = 0, lastpos = -1;
		if (length==0) return;
		while ((pos += in.read(array,offset+pos,length-pos)) < length) if (pos<=lastpos) throw new EOFException(); else lastpos = pos;
	}

//	private java.awt.Component component;

	public Mod(InputStream is, java.awt.Component c) throws Exception {
//		component = c;


		if (is==null) throw new FileNotFoundException();

		BufferedInputStream in = new BufferedInputStream(is);

		byte buffer[] = new byte[2048];
		int b;

		in.mark(2048);
//		in.skip(1080);
		readFully(in,buffer,0,1080);
		readFully(in,buffer,0,4);

		channels = 4;
		int numSamples = 31;

		String id=null;
		if (buffer[0] == 'M' && buffer[1] == '.' && buffer[2] == 'K' && buffer[3] == '.') channels = 4;
		else if (buffer[0] == '6' && buffer[1] == 'C' && buffer[2] == 'H' && buffer[3] == 'N') channels = 6;
		else if (buffer[0] == '8' && buffer[1] == 'C' && buffer[2] == 'H' && buffer[3] == 'N') channels = 8;
		else if (buffer[2] == 'C' && buffer[3] == 'H') {
			String chString = new String(buffer,0,2);
			try {
				channels = Integer.parseInt(chString);
			} catch (NumberFormatException nfe) {
				id = new String(buffer,0,4);
				throw new Exception("Bad mod file, "+id);
			}
		} else {
			id = new String(buffer,0,4);
			numSamples = 15;
		}


		in.reset();


		readFully(in,buffer,0,20);
		songName = new String(buffer,0,20);


		samples = new Sample[numSamples];
		for (int i = 0; i < samples.length; i++) {
			samples[i] = new Sample();

			readFully(in,buffer,0,22);
			samples[i].name = new String(buffer,0,22);

			readFully(in,buffer,0,2);
			samples[i].length = 2*(((buffer[0]&0xFF)<<8) | (buffer[1]&0xFF));

			b = in.read();
//			if ((b&0x08) > 0)
//				samples[i].finetune = (byte) (b|0xF0);
//			else
//				samples[i].finetune = (byte) b;
			if (b>7)
				samples[i].finetune = (byte) (b-16);
			else
				samples[i].finetune = (byte) b;

			b = in.read();
			samples[i].volume = (byte) b;
			if (samples[i].volume < 0) samples[i].volume = 0;
			else if (samples[i].volume > 64) samples[i].volume = 64;

			readFully(in,buffer,0,2);
			samples[i].loopStart = 2*(((buffer[0]&0xFF)<<8) | (buffer[1]&0xFF));

			readFully(in,buffer,0,2);
			samples[i].loopLength = 2*(((buffer[0]&0xFF)<<8) | (buffer[1]&0xFF));

			if (samples[i].loopLength<=2) samples[i].loopLength = 0;
		}

		b = in.read();
		songLength = b;

		b = in.read();

		orders = new byte[128];
		readFully(in,orders,0,128);

		int maxPatternNum = 0;
		for (int i = 0; i < 128; i++)
			if (orders[i] > maxPatternNum) maxPatternNum = orders[i];

		if (numSamples!=15)
			readFully(in,buffer,0,4);




		patterns = new Note[maxPatternNum+1][];

		buffer = new byte[channels*4*64];

		for (int i = 0; i < maxPatternNum+1; i++) {
			patterns[i] = new Note[channels*64];
//			int pos = 0, lastpos = -1;
//			int length = channels*4*64;
//			while ((pos += in.read(buffer,pos,length-pos)) < length) if (pos<=lastpos) break; else lastpos = pos;
			readFully(in,buffer,0,channels*4*64);
			for (int j = 0; j < channels*64; j++) {
				patterns[i][j] = new Note();
				patterns[i][j].note = (short) (((buffer[j*4+0]&0x0F)<<8)|(buffer[j*4+1]&0xFF));
				patterns[i][j].note = (short) getNote(patterns[i][j].note);
				patterns[i][j].sample = (byte) ((buffer[j*4+0]&0xF0) | ((buffer[j*4+2]&0xFF)>>4));
				patterns[i][j].effect = (byte) (buffer[j*4+2]&0x0F);
				patterns[i][j].parameter = (short) (buffer[j*4+3]&0xFF);
			}
		}

		for (int i = 0; i < samples.length; i++) {
			samples[i].data = new byte[samples[i].length];
//			int pos = 0, lastpos = -1;
//			int length = samples[i].length;
//			if (length==0) continue;
//			while ((pos += in.read(samples[i].data,pos,length-pos)) < length) if (pos<=lastpos) break; else lastpos = pos;
//			while ((total += in.read(samples[i].data,total,max-total)) < max) if (in.available()==0) break;
			readFully(in,samples[i].data,0,samples[i].length);
		}

		if (id!=null && in.available()!=0)
			throw new Exception("Bad mod file, "+id);


		started = false;

		listener = null;
		listenChannel = -1;


		ad = Audio.init(this,c);
	}

	private static final int table0[] = {
		856,808,762,720,678,640,604,570,538,508,480,453,
		428,404,381,360,339,320,302,285,269,254,240,226,
		214,202,190,180,170,160,151,143,135,127,120,113,
	};
	private static final int table1[] = {
		850,802,757,715,674,637,601,567,535,505,477,450,
		425,401,379,357,337,318,300,284,268,253,239,225,
		213,201,189,179,169,159,150,142,134,126,119,113,
	};
	private static final int table2[] = {
		844,796,752,709,670,632,597,563,532,502,474,447,
		422,398,376,355,335,316,298,282,266,251,237,224,
		211,199,188,177,167,158,149,141,133,125,118,112,
	};
	private static final int table3[] = {
		838,791,746,704,665,628,592,559,528,498,470,444,
		419,395,373,352,332,314,296,280,264,249,235,222,
		209,198,187,176,166,157,148,140,132,125,118,111,
	};
	private static final int table4[] = {
		832,785,741,699,660,623,588,555,524,495,467,441,
		416,392,370,350,330,312,294,278,262,247,233,220,
		208,196,185,175,165,156,147,139,131,124,117,110,
	};
	private static final int table5[] = {
		826,779,736,694,655,619,584,551,520,491,463,437,
		413,390,368,347,328,309,292,276,260,245,232,219,
		206,195,184,174,164,155,146,138,130,123,116,109,
	};
	private static final int table6[] = {
		820,774,730,689,651,614,580,547,516,487,460,434,
		410,387,365,345,325,307,290,274,258,244,230,217,
		205,193,183,172,163,154,145,137,129,122,115,109,
	};
	private static final int table7[] = {
		814,768,725,684,646,610,575,543,513,484,457,431,
		407,384,363,342,323,305,288,272,256,242,228,216,
		204,192,181,171,161,152,144,136,128,121,114,108,
	};
/*	private static final int table_8[] = {
		907,856,808,762,720,678,640,604,570,538,508,480,
		453,428,404,381,360,339,320,302,285,269,254,240,
		226,214,202,190,180,170,160,151,143,135,127,120,
	};
	private static final int table_7[] = {
		900,850,802,757,715,675,636,601,567,535,505,477,
		450,425,401,379,357,337,318,300,284,268,253,238,
		225,212,200,189,179,169,159,150,142,134,126,119,
	};
	private static final int table_6[] = {
		894,844,796,752,709,670,632,597,563,532,502,474,
		447,422,398,376,355,335,316,298,282,266,251,237,
		223,211,199,188,177,167,158,149,141,133,125,118,
	};
	private static final int table_5[] = {
		887,838,791,746,704,665,628,592,559,528,498,470,
		444,419,395,373,352,332,314,296,280,264,249,235,
		222,209,198,187,176,166,157,148,140,132,125,118,
	};
	private static final int table_4[] = {
		881,832,785,741,699,660,623,588,555,524,494,467,
		441,416,392,370,350,330,312,294,278,262,247,233,
		220,208,196,185,175,165,156,147,139,131,123,117,
	};
	private static final int table_3[] = {
		875,826,779,736,694,655,619,584,551,520,491,463,
		437,413,390,368,347,328,309,292,276,260,245,232,
		219,206,195,184,174,164,155,146,138,130,123,116,
	};
	private static final int table_2[] = {
		868,820,774,730,689,651,614,580,547,516,487,460,
		434,410,387,365,345,325,307,290,274,258,244,230,
		217,205,193,183,172,163,154,145,137,129,122,115,
	};
	private static final int table_1[] = {
		862,814,768,725,684,646,610,575,543,513,484,457,
		431,407,384,363,342,323,305,288,272,256,242,228,
		216,203,192,181,171,161,152,144,136,128,121,114,
	};
*/
	private static final int tables[][] = {
//		table_8, table_7, table_6, table_5, table_4, table_3, table_2, table_1,
		table0, table1, table2, table3, table4, table5, table6, table7
	};

	private static int sineTable[] = {
		  0, 24, 49, 74, 97,120,141,161,
		180,197,212,224,235,244,250,253,
		255,253,250,244,235,224,212,197,
		180,161,141,120, 97, 74, 49, 24,
	};



	private final int getNote(int period) {
		if (period==0) return 0;

		for (int i = 0; i < table0.length; i++) {
			if (table0[i] == period) return (i+1)*8;
			if (table0[i]+2 >= period && table0[i]-2 <= period) return (i+1)*8;
		}

		return 0;

/*
		int minDist = 1000;
		int note = 0;
		for (int i = 0; i < 16; i++) {
			int table[] = tables[i];
			int finetune = i-8;
			for (int j = 0; j < table.length; j++) {
				int dist = Math.abs(period-table[j]);
				if (dist<minDist) {
					minDist = dist;
					note = (j+1)*8 + finetune;
				}
			}
		}

		return note;
*/	}


	private boolean running;
	private boolean started;
	private Thread thread;

	private ModListener listener;
	private int listenChannel;

	private int speed;
	private int tick;
	private int row;
	private int bpm;

	private int tickLength;
	private int tickPos;

	private int orderNr;
	private int currentPattern;
	private int patternOffset;

	private int lastInstrument[];
	private int origVolume[];
	private int volume[];
	private int tone[];
	private int amiga[];
	private int frequency[];
	private int pan[];

	private int portaSpeed[];
	private int portaNote[];
	private int vibratoPos[];
	private boolean vibratoNeg[];
	private int vibratoPar[];
	private int vibratoType[];
	private int tremoloPos[];
	private boolean tremoloNeg[];
	private int tremoloPar[];
	private int tremoloType[];
	private boolean glissando[];
	private int loopRow;
	private int loopNum;
	private int patternDelay;

	private AudioDevice ad;
	private int outFreq;
	private int frameSize;
	private int lastPos[];

	private boolean fade;
	private int fadePos;

	private final void timer() {
		tick++;

		if (tick>=speed) {
			if (patternDelay==0) {
				updateRow();
				row++;
			} else patternDelay--;
			tick = 0;
		} else {
			updateEffect();
		}

	}

	private static byte oneByte[] = new byte[1];
	public final int read() {
		read(oneByte,0,1);
		return oneByte[0];
	}

	public final int read(byte buffer[], int offset, int length) {

		boolean is16Bit = ad.is16Bit();
		boolean isStereo = ad.isStereo();
		int bufferPos = offset;

//		int numSamples = length/((is16Bit?2:1)*(isStereo?2:1));
		int numSamples = length/frameSize;

		for (int i = 0; i < numSamples; i++) {
			if (tickPos>=tickLength) {
				tickPos = 0;

				for (int j = 0; j < channels; j++) {
					lastPos[j] += tickLength*frequency[j]/outFreq;
				}

				timer();
			}

			int r = 0;
			int l = 0;

			if (running) {

				for (int j = 0; j < channels; j++) {
					int sampleNum = lastInstrument[j];
					int samplePos = lastPos[j] + tickPos*frequency[j]/outFreq;

					if (sampleNum<0) continue;

					if (samples[sampleNum].loopLength>0) {
						if (samplePos >= samples[sampleNum].loopStart + samples[sampleNum].loopLength) {
							samplePos -= samples[sampleNum].loopLength;
							lastPos[j] -= samples[sampleNum].loopLength;
						}
					}

					if (samplePos >= samples[sampleNum].data.length) continue;


					int sample = (samples[sampleNum].data[samplePos]*volume[j])<<1;

/*					int s1 = samples[sampleNum].data[samplePos];
					int s2 = samples[sampleNum].data[samplePos+1];
					float fraq = ((float)i*frequency[j]/outFreq) - (int)(i*frequency[j]/outFreq);
					int sample = (((int)(s1 + fraq*(s2-s1)))*volume[j])<<1;
*/

					if (pan[j]<127) {
						l += sample;
						r += pan[j]*sample/127;
					} else {
						l += (255-pan[j])*sample/127;
						r += sample;
					}

//					l += (255-pan[j])*sample/255;
//					r += pan[j]*sample/255;
				}
			}



//			int r = (channelData[1]+channelData[2])&0x0000FFFF;
//			int l = (channelData[0]+channelData[3])&0x0000FFFF;

//			buffer[i] = (l<<16) | (r&0x0000FFFF);

			if (fade) {
				if (fadePos>=outFreq) {
					started = false;
					running = false;
					l = 0;
					r = 0;
				} else {
					l = l*(outFreq-fadePos)/(outFreq);
					r = r*(outFreq-fadePos)/(outFreq);
					fadePos++;
				}
			}

			if (isStereo) {
				if (l>0x7FFF) l = 0x7FFF;
				else if (l<-0x7FFF) l = -0x7FFF;
				if (r>0x7FFF) r = 0x7FFF;
				else if (r<-0x7FFF) r = -0x7FFF;

				if (is16Bit) {
					buffer[bufferPos++] = (byte) ((l&0x00FF));
					buffer[bufferPos++] = (byte) ((l&0xFF00)>>8);
					buffer[bufferPos++] = (byte) ((r&0x00FF));
					buffer[bufferPos++] = (byte) ((r&0xFF00)>>8);
				} else {
					buffer[bufferPos++] = (byte) ((l&0xFF00)>>8);
					buffer[bufferPos++] = (byte) ((r&0xFF00)>>8);
				}
			} else {
				int sample = (l>>1)+(r>>1);
				if (sample>0x7FFF) sample = 0x7FFF;
				else if (sample<-0x7FFF) sample = -0x7FFF;

				if (is16Bit) {
					buffer[bufferPos++] = (byte) ((sample&0x00FF));
					buffer[bufferPos++] = (byte) ((sample&0xFF00)>>8);
				} else {
					buffer[bufferPos++] = (byte) (((sample&0xFF00)>>8)&0xFF);
				}
			}


//			buffer[i] = (((channelData[0]+channelData[3])&0x0000FFFF)<<16) | ((channelData[1]+channelData[2])&0x0000FFFF);
			// l<<16 | r;


			tickPos++;
		}

		if (!running) stop();

		return length;
	}

	private final void nextPattern() {
		row = 0;
		orderNr++;
		if (orderNr>=songLength) {
			running = false;
			started = false;
			return;
		}
		if (listener!=null) listener.newPattern(orderNr);
		currentPattern = orders[orderNr];
	}

	private final void updateRow() {
		if (row>=64) nextPattern();

		if (!running) return;

		if (listener!=null) listener.newRow(row,currentPattern,orderNr,songLength);

		patternOffset = row*channels;

		boolean jumpToNext = false;
		int nextRow = 0;

		boolean jumpToPattern = false;
		int nextPattern = 0;

//		System.out.print(row+" "+currentPattern+" "+orderNr);

		for (int i = 0; i < channels; i++) {
			Note n = patterns[currentPattern][patternOffset+i];

//			System.out.print(" "+n.note+" "+n.sample+" "+n.effect+" "+n.parameter);

			if (n.sample>0) {
				if (n.sample<=samples.length) {
					lastInstrument[i] = n.sample-1;
					origVolume[i] = samples[lastInstrument[i]].volume;
				}
			}

			if (n.note > 0) {
				if (n.effect != 0x3 && n.effect != 0x5 && (n.effect != 0xE || ((n.parameter&0xF0)>>4) != 0xD)) {
					lastPos[i] = 0;
					tone[i] = n.note + samples[lastInstrument[i]].finetune;
					amiga[i] = toneToAmiga(tone[i]);
					frequency[i] = toFreq(amiga[i]);
					if (listener!=null && i==listenChannel) listener.channelHit();
					if (vibratoType[i]>=4) {
						vibratoPos[i] = 0;
						vibratoNeg[i] = false;
					}
					if (tremoloType[i]>=4) {
						tremoloPos[i] = 0;
						tremoloNeg[i] = false;
					}
				}
			}



			if (n.effect!=0 || n.parameter!=0) {

				switch (n.effect) {
					case 0xD:
						// Pattern Break
						if (n.parameter < 0x40 && n.parameter >= 0)
							nextRow = n.parameter;
						else
							nextRow = 0;
						jumpToNext = true;
						break;
					case 0xF:
						// Set speed
						if (n.parameter <= 0x1F && n.parameter >= 0x00) {
							speed = n.parameter;
						} else if (n.parameter >= 0x20 && n.parameter <= 0xFF) {
							bpm = n.parameter;
							updateSpeed();
						}
						break;
					case 0xC:
						// Set volume
						if (n.parameter < 0)
							origVolume[i] = 0;
						else if (n.parameter > 0x40)
							origVolume[i] = 0x40;
						else
							origVolume[i] = n.parameter;
						break;
					case 0xB:
						// Jump To Pattern
						if (n.parameter >= 0 && n.parameter < songLength) {
//							jumpToPattern = true;
							nextPattern = n.parameter;
						}
						break;
					case 0x3:
						// Porta To Note
						if (n.note>0)
							portaNote[i] = toneToAmiga(n.note + samples[lastInstrument[i]].finetune);
						if (n.parameter>0)
							portaSpeed[i] = n.parameter;
						break;
					case 0x4:
						// Vibrato
						if (n.parameter>0)
							vibratoPar[i] = n.parameter;
						break;
					case 0x8:
						// Pan
						if (n.parameter<=0x80) {
							pan[i] = 255*n.parameter/0x80;
						} else if (n.parameter==0xA4) {
						}
						break;
					case 0x7:
						// Tremolo
						if (n.parameter>0)
							tremoloPar[i] = n.parameter;
						break;
					case 0x9:
						// Sample offset
						lastPos[i] += n.parameter<<8;
						if (lastPos[i]>=samples[lastInstrument[i]].length) lastPos[i] -= n.parameter<<8;
						break;
					case 0xE:
						int subtype = (n.parameter&0xF0)>>4;
						int parameter = n.parameter&0x0F;
						switch (subtype) {
							case 0x0:
								// Set filter
								break;
							case 0x1:
								// Fine Porta Up
								amiga[i] -= parameter;
								break;
							case 0x2:
								// Fine Porta Down
								amiga[i] += parameter;
								break;
							case 0x3:
								// Glissando Control
								if (parameter==0x1) glissando[i] = true;
								else glissando[i] = false;
								break;
							case 0x4:
								// Vibrato Waveform
								vibratoType[i] = parameter;
								break;
							case 0x5:
								// Set Finetune
								if (parameter>7)
									samples[lastInstrument[i]].finetune = (byte) (parameter-16);
								else
									samples[lastInstrument[i]].finetune = (byte) parameter;
								break;
							case 0x6:
								// Pattern Loop
								if (parameter==0) loopRow = row;
								else {
									if (loopNum==0) loopNum = parameter;
									else loopNum--;
									if (loopNum>0) row = loopRow-1;
								}
								break;
							case 0x7:
								// Tremolo Waveform
								tremoloType[i] = parameter;
								break;
							case 0x8:
								// 16 Pos Panning
								pan[i] = 255*parameter/15;
								break;
							case 0xA:
								// Fine VolSlide Up
								origVolume[i] += parameter;
								if (origVolume[i]>0x40) origVolume[i] = 0x40;
								break;
							case 0xB:
								// Fine VolSlide Down
								origVolume[i] -= parameter;
								if (origVolume[i]<0) origVolume[i] = 0;
								break;
							case 0xE:
								// Pattern Delay
								patternDelay = parameter;
								break;
							case 0xF:
								// Invert Loop
								break;
						}
						break;
				}

				frequency[i] = toFreq(amiga[i]);
				volume[i] = origVolume[i];
			}

		}
//		System.out.println();

		if (jumpToNext) {
			nextPattern();
			row = nextRow-1;
		}
		if (jumpToPattern) {
			orderNr = nextPattern;
			currentPattern = orders[orderNr];
			row = -1;
		}

//		updateVibratoTremolo();
	}

	private final void updateEffect() {
		for (int i = 0; i < channels; i++) {

			Note n = patterns[currentPattern][patternOffset+i];

			if (n.effect!=0 || n.parameter!=0) {

				if (n.effect == 0xA || n.effect == 0x5 || n.effect == 0x6) {
					// Volume Slide
					if ((n.parameter&0xF0)==0 && (n.parameter&0x0F)>0)
						origVolume[i] -= n.parameter&0x0F;
					else if ((n.parameter&0xF0)>0 && (n.parameter&0xF0)==0)
						origVolume[i] += (n.parameter&0xF0)>>4;

					if (origVolume[i] > 0x40)
						origVolume[i] = 0x40;
					else if (origVolume[i] < 0)
						origVolume[i] = 0;

				}

				switch (n.effect) {
					case 0x1:
						// Porta Up
						amiga[i] -= n.parameter;
						// 54
						if (amiga[i]<5) amiga[i] = 5;
						break;
					case 0x2:
						// Porta Down
						amiga[i] += n.parameter;
						break;
					case 0x3:
					case 0x5:
						// Porta To Note
						if (amiga[i] > portaNote[i]) {
							if (!glissando[i]) {
								amiga[i] -= portaSpeed[i];
							} else {
								tone[i] += 8;
								amiga[i] = toneToAmiga(tone[i]);
							}
							if (amiga[i] < portaNote[i]) amiga[i] = portaNote[i];
						} else if (amiga[i] < portaNote[i]) {
							if (!glissando[i]) {
								amiga[i] += portaSpeed[i];
							} else {
								tone[i] -= 8;
								amiga[i] = toneToAmiga(tone[i]);
							}
							if (amiga[i] > portaNote[i]) amiga[i] = portaNote[i];
						}
						break;
					case 0x0:
						// Arpeggio
						switch (tick%3) {
							case 0:
								amiga[i] = toneToAmiga(tone[i]);
								break;
							case 1:
								amiga[i] = toneToAmiga(tone[i]+8*((n.parameter&0xF0)>>4));
								break;
							case 2:
								amiga[i] = toneToAmiga(tone[i]+8*(n.parameter&0x0F));
								break;
						}
						break;

					case 0xE:
						int subtype = (n.parameter&0xF0)>>4;
						int parameter = n.parameter&0x0F;
						switch (subtype) {
							case 0x9:
								// Retrig Note
								if (tick%parameter==0) lastPos[i] = 0;
								break;
							case 0xC:
								// Cut Note
								if (tick==parameter) origVolume[i] = 0;
								break;
							case 0xD:
								// Delay Note
								if (tick==parameter) {
									lastPos[i] = 0;
									tone[i] = n.note + samples[lastInstrument[i]].finetune;
									amiga[i] = toneToAmiga(tone[i]);
									frequency[i] = toFreq(amiga[i]);
									if (listener!=null && i==listenChannel) listener.channelHit();
									if (vibratoType[i]>=4) {
										vibratoPos[i] = 0;
										vibratoNeg[i] = false;
									}
									if (tremoloType[i]>=4) {
										tremoloPos[i] = 0;
										tremoloNeg[i] = false;
									}
								}
								break;
						}
						break;
				}

			}

			volume[i] = origVolume[i];
			frequency[i] = toFreq(amiga[i]);

			// If using arpeggio, reset amiga-value, to get the real tone after the effect
			if (n.effect==0x0 && n.parameter!=0x00) amiga[i] = toneToAmiga(tone[i]);


			if (n.effect==0x4 || n.effect==0x6) {
				// Vibrato
				int depth = vibratoPar[i]&0x0F;
				int speed = (vibratoPar[i]&0xF0)>>4;
				int delta = 0;
				switch (vibratoType[i]) {
					case 0:
					case 4:
						// Sine wave
						delta = sineTable[vibratoPos[i]];
						break;
					case 1:
					case 5:
						// Ramp down
						if (!vibratoNeg[i]) delta = (-8*vibratoPos[i]);
						else delta = -(255 - (8*vibratoPos[i]));
						break;
					case 2:
					case 6:
						// Square wave
						delta = 255;
						break;
					case 3:
					case 7:
						// Random
						delta = (int)(255-511*Math.random());
						break;
				}

				delta = (depth*delta)>>7;

				if (!vibratoNeg[i]) frequency[i] = toFreq(amiga[i]+delta);
				else frequency[i] = toFreq(amiga[i]-delta);

				vibratoPos[i] += speed;
				if (vibratoPos[i]>=32) {
					vibratoPos[i] -= 32;
					vibratoNeg[i] = (vibratoNeg[i]) ? false : true;
				}
			} else if (n.effect==0x7) {
				// Tremolo
				int depth = tremoloPar[i]&0x0F;
				int speed = (tremoloPar[i]&0xF0)>>4;
				int delta = 0;
				switch (tremoloType[i]) {
					case 0:
					case 4:
						// Sine wave
						delta = sineTable[tremoloPos[i]];
						break;
					case 1:
					case 5:
						// Ramp down
						if (!tremoloNeg[i]) delta = (-8*tremoloPos[i]);
						else delta = -(255 - (8*tremoloPos[i]));
						break;
					case 2:
					case 6:
						// Square wave
						delta = 255;
						break;
					case 3:
					case 7:
						// Random
						delta = (int)(255-511*Math.random());
						break;
				}

				delta = (depth*delta)>>6;

				if (!tremoloNeg[i]) {
					volume[i] = origVolume[i]+delta;
					if (volume[i]>0x40) volume[i] = 0x40;
				} else {
					volume[i] = origVolume[i]-delta;
					if (volume[i]<0) volume[i] = 0;
				}

				tremoloPos[i] += ((tremoloPar[i]&0xF0)>>4);
				if (tremoloPos[i]>=32) {
					tremoloPos[i] -= 32;
					tremoloNeg[i] = (tremoloNeg[i]) ? false : true;
				}
			}

		}

	}


	private final int toneToAmiga(int tone) {
		int pos = (tone/8)-1;
		int finetune = (tone%8);//+8;

/*		if (pos<0) {
			finetune -= 8;
			pos++;
		}
*/
		try {
			return tables[finetune][pos];
		} catch (ArrayIndexOutOfBoundsException aioobe) {
			return 0;
		}
	}

	private final int toFreq(int amiga) {
		if (amiga<=0) return 0;

//		return (int) (7093789.2 / (amiga*2));
		return (int) (7159090.5 / (amiga*2));
	}

	private final void updateSpeed() {
		int hz = 2*bpm/5;
		int length = 1000/hz;

		frameSize = 1;
		if (ad.isStereo()) frameSize *= 2;
		if (ad.is16Bit()) frameSize *= 2;

		tickLength = outFreq/hz;
	}

	private final void playInit() {
		lastInstrument = new int[channels];
		volume = new int[channels];
		origVolume = new int[channels];
		tone = new int[channels];
		amiga = new int[channels];
		frequency = new int[channels];
		pan = new int[channels];
		lastPos = new int[channels];


		portaSpeed = new int[channels];
		portaNote = new int[channels];

		vibratoPos = new int[channels];
		vibratoNeg = new boolean[channels];
		vibratoPar = new int[channels];
		vibratoType = new int[channels];

		tremoloPos = new int[channels];
		tremoloNeg = new boolean[channels];
		tremoloPar = new int[channels];
		tremoloType = new int[channels];

		glissando = new boolean[channels];
		loopRow = 0;
		loopNum = 0;
		patternDelay = 0;



		for (int i = 0; i < channels; i++) {
			int n = i%4;
			if (n==0 || n==3) pan[i] = 64;
			else pan[i] = 192;

			lastInstrument[i] = -1;
			glissando[i] = false;
		}

		running = true;
		started = true;
		fade = false;

		speed = 6;
//		bpm = 125;
		bpm = 125;
		updateSpeed();

		tick = speed;
		row = 64;
		orderNr = -1;

		tickPos = tickLength;
	}

	public final void play() {
		if (ad==null) return;

		outFreq = ad.getSampleRate();
		if (!started) playInit();

		ad.start();
	}

	public final boolean isPlaying() {
		return running;
	}

	public final void stop() {
		ad.stop();
	}

	public final void fadeOut() {
		fade = true;
		fadePos = 0;
	}

	public final void addListener(ModListener ml, int c) {
		listener = ml;
		listenChannel = c;
	}

}


